package ics.uci.edu.Calico.Structures;

import ics.uci.edu.Calico.models.RelationshipModel;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

public class RelationshipController {

	public static void thicken(RelationshipModel r) {
		r.thickness++;
	}
	
	public static void thin(RelationshipModel r) {
		r.thickness--;
	}
	
	public static void translate(RelationshipModel r, int hook, int deltaX, int deltaY) {
		if (hook == r.hook1){
			r.setX1(r.getX1() + deltaX);
			r.setY1(r.getY1() + deltaY);
		}
		else if (hook == r.hook2){
			r.setX2(r.getX2() + deltaX);
			r.setY2(r.getY2() + deltaY);
		}
		else
			System.err.println("RelationshipModel:translate - invalid hook passed");
	}
	
	public static void translate(RelationshipModel r, int deltaX, int deltaY) {
		r.setX1(r.getX1() + deltaX);
		r.setY1(r.getY1() + deltaY);
		r.setX2(r.getX2() + deltaX);
		r.setY2(r.getY2() + deltaY);
	}

	public static boolean connectedTo(RelationshipModel model, int id) {
		if (model.hook1 == id) 
			return true;
		else if (model.hook2 == id)
			return true;
		
		return false;
	}

	public static double getThickness(RelationshipModel rel) {
		return rel.thickness;
	}
	
	public static RelationshipModel copy(RelationshipModel r) {
		RelationshipModel ret = new RelationshipModel(r.hook1, r.hook2, r.getX1(), 
				r.getY1(), r.getX2(), r.getY2(), r.id);
		ret.thickness = r.thickness;
		ret.setColor(r.getColor());
		return ret;
	}
	
	public static boolean intersects(RelationshipModel r, Line2D line) {
		Line2D rel = new Line2D.Double(new Point2D.Double(r.getX1(), r.getY1()),
				new Point2D.Double(r.getX2(), r.getY2()));
		return rel.intersectsLine(line);
	}
	
	public static Point2D getIntersectingPoint(RelationshipModel r, Line2D line) {
		return getIntersection(new Line2D.Double(r.getX1(), r.getY1(), r.getX2(), r.getY2()), line);
	}
	
	/**
	 * Taken from: http://www.koders.com/java/fid1182C9330FDC66A60312A33932E82F09B5C4CA9F.aspx
	 * 
	 * Returns the intersection point of two lines.
	 *
	 * @param   line1   First line
	 * @param   line2   Second line
	 * @return  The Point object where the two lines intersect. This method
	 * returns null if the two lines do not intersect.
	 * @throws  <tt>MultipleIntersectionException</tt> when the two lines
	 * have more than one intersection point.
	 */
	public static Point2D getIntersection(Line2D line1, Line2D line2) {
		double dyline1, dxline1;
		double dyline2, dxline2, e, f;
		double x1line1, y1line1, x2line1, y2line1;
		double x1line2, y1line2, x2line2, y2line2;
	
	    if (!line1.intersectsLine(line2))
	      return null;
	
		/* first, check to see if the segments intersect by parameterization
		 on s and t; if s and t are both between [0,1], then the
		 segments intersect */
		x1line1 = line1.getX1();
		y1line1 = line1.getY1();
		x2line1 = line1.getX2();
		y2line1 = line1.getY2();
		
		x1line2 = line2.getX1();
		y1line2 = line2.getY1();
		x2line2 = line2.getX2();
		y2line2 = line2.getY2();
		
		/* check to see if the segments have any endpoints in common. If they do,
		   then return the endpoints as the intersection point */
		if ((x1line1==x1line2) && (y1line1==y1line2))  {
		  return (new Point2D.Double(x1line1, y1line1));
		}
		if ((x1line1==x2line2) && (y1line1==y2line2)) {
		  return (new Point2D.Double(x1line1, y1line1));
		}
		if ((x2line1==x1line2) && (y2line1==y1line2)) {
		  return (new Point2D.Double(x2line1, y2line1));
		}
		if ((x2line1==x2line2) && (y2line1==y2line2)) {
		  return (new Point2D.Double(x2line1, y2line1));
		}
		
		dyline1 = -(y2line1 - y1line1);
		dxline1 = x2line1 - x1line1;
		
		dyline2 = -(y2line2 - y1line2);
		dxline2 = x2line2 - x1line2;
		
		e = -(dyline1 * x1line1) - (dxline1 * y1line1);
		f = -(dyline2 * x1line2) - (dxline2 * y1line2);
		
		/* compute the intersection point using
		  ax+by+e = 0 and cx+dy+f = 0
		
		  If there is more than 1 intersection point between two lines,
		*/
		//if( (dyline1 * dxline2 - dyline2 * dxline1) == 0 )
		//    throw new MultipleIntersectionException();
	    return (new Point2D.Double(
		      (-(e * dxline2 - dxline1 * f)/(dyline1 * dxline2 - dyline2 * dxline1)),
		      (-(dyline1 * f - dyline2 * e)/(dyline1 * dxline2 - dyline2 * dxline1))));
	}
	
	/**
	 * Returns the edge of RelationshipModel that is closest to Point p. 
	 * @param r
	 * @param p
	 * @return 0 for the first edge, 1 for the second edge.
	 */
	public static int getClosestEdge(RelationshipModel r, Point2D p) {
		if (distance(new Point2D.Double(r.getX1(), r.getY1()), p) >
			distance(new Point2D.Double(r.getX2(), r.getY2()), p))
			return 0;
		else
			return 1;
	}
	
	public static double distance(Point2D p1, Point2D p2) {
		return Math.sqrt(Math.pow(p2.getX()-p1.getX(), 2) + Math.pow(p2.getY()-p1.getY(), 2));
	}
	
	/**
	 * TODO: Give credit to the writers of org.tigris.gef.base.Geometry
	 * Returns a point on the relationship line that is closest to point p.
	 * @param r
	 * @param p
	 * @return
	 */
	public static Point2D ptClosest(RelationshipModel r, Point2D p) {
		Point2D ret;
		Point2D p1 = new Point2D.Double(r.getX1(), r.getY1());
		Point2D p2 = new Point2D.Double(r.getX2(), r.getY2());
		double retY;
		double retX;
		
		
	        // segment is a point
	        if(p1.getY() == p2.getY() && p1.getX() == p2.getX()) {
	        	return new Point2D.Double(p1.getX(), p1.getY());
	        }
	        // segment is horizontal
	        if(p1.getY() == p2.getY()) {
	        	return new Point2D.Double(getMedian(p1.getX(), p2.getX(), p.getX()), p1.getY());
	        }
	        // segment is vertical
	        if(p1.getX() == p2.getX()) {
	        	return new Point2D.Double(p1.getX(), getMedian(p1.getY(), p2.getY(), p.getY()));
	        }
	        double dx = p2.getX() - p1.getX();
	        double dy = p2.getY() - p1.getY();
	        retX = dy * (dy * p1.getX() - dx * (p1.getY() - p.getY())) + dx * dx * p.getX();
	        retX = retX / (dx * dx + dy * dy);
	        retY = (dx * (p.getX() - retX)) / dy + p.getY();

	        if (p2.getX() > p1.getX()) {
	            if (retX > p2.getX()) {
	                retX = p2.getX();
	                retY = p2.getY();
	            } else if (retX < p1.getX()) {
	                retX = p1.getX();
	                retY = p1.getY();
	            }
	        } else {
	            if (retX < p2.getX()) {
	                retX = p2.getX();
	                retY = p2.getY();
	            } else if (retX > p1.getX()) {
	                retX = p1.getX();
	                retY = p1.getY();
	            }
	        }
	        return new Point2D.Double(retX, retY);
	}
	
	/**
	 * Returns the median of three points.
	 * @param a
	 * @param b
	 * @param c
	 * @return
	 */
	public static double getMedian(double x, double y, double z) {
		if (x <= y)
			if (y <= z)
				return y;
			else if (z <= x)
				return x;
			else
				return z;
		if (y >= z)
			return y;
		else if (z >= x)
			return x;
		else
			return z;
	}
	
	/**
	 * Returns true if Point2D p is above the point on the relationship line closest 
	 * 	to Point2D p. Keep in mind when using this that the swing libraries consider 0,0
	 * 	to be the upper left corner of the screen, so what visually is considered above
	 *  is actually below. >_>
	 * @param r
	 * @param p
	 * @return
	 */
	public static boolean isAbove(RelationshipModel r, Point2D p) {
		Point2D closestPt = RelationshipController.ptClosest(r, p);
		if (closestPt.getY() > p.getY())
			return true;
		else
			return false;
	}
	
	/**
	 * Taken from http://www.uk-dave.com/bytes/java/angle_between_points.html
	 * @param d
	 * @param e
	 * @param f
	 * @param g
	 * @return
	 */
    public static double calcAngle(double d, double e, double f, double g)
    {
        double dx = f-d;
        double dy = g-e;
        double angle=0.0d;
 
        // Calculate angle
        if (dx == 0.0)
        {
            if (dy == 0.0)
                angle = 0.0;
            else if (dy > 0.0)
                angle = Math.PI / 2.0;
            else
                angle = Math.PI * 3.0 / 2.0;
        }
        else if (dy == 0.0)
        {
            if  (dx > 0.0)
                angle = 0.0;
            else
                angle = Math.PI;
        }
        else
        {
            if  (dx < 0.0)
                angle = Math.atan(dy/dx) + Math.PI;
            else if (dy < 0.0)
                angle = Math.atan(dy/dx) + (2*Math.PI);
            else
                angle = Math.atan(dy/dx);
        }
 
        // Convert to degrees
        angle = angle * 180 / Math.PI;
 
        // Return
        return angle;
    }
	
	public static Polygon getArrowhead(RelationshipModel r) {
		Point2D head = new Point2D.Double(r.x2, r.y2),
			tail = new Point2D.Double(r.x1, r.y1);
		
		//get angle
		double angle = calcAngle(r.x2, r.y2, r.x1, r.y1);
		
		
		
		
		return null;
	}
	
	public static Polygon getArrow(int x1, int y1, int x2, int y2, int headsize, int difference, double factor) {
		int[] crosslinebase = getArrowHeadLine(x1, y1, x2, y2, headsize);
		int[] headbase = getArrowHeadLine(x1, y1, x2, y2, headsize - difference);
		int[] crossline = getArrowHeadCrossLine(crosslinebase[0], crosslinebase[1], x2, y2, factor);

		Polygon head = new Polygon();

		head.addPoint(headbase[0], headbase[1]);
		head.addPoint(crossline[0], crossline[1]);
		head.addPoint(x2, y2);
		head.addPoint(crossline[2], crossline[3]);
		head.addPoint(headbase[0], headbase[1]);
		head.addPoint(x1, y1);

		return head;
	}

	private static int[] getArrowHeadLine(int xsource, int ysource,int xdest,int ydest, int distance) {
		int[] arrowhead = new int[2];
		int headsize = distance;

		double stretchfactor = 0;
		stretchfactor = 1 - (headsize/(Math.sqrt(((xdest-xsource)*(xdest-xsource))+((ydest-ysource)*(ydest-ysource)))));

		arrowhead[0] = (int) (stretchfactor*(xdest-xsource))+xsource;
		arrowhead[1] = (int) (stretchfactor*(ydest-ysource))+ysource;

		return arrowhead;
	}

	private static int[] getArrowHeadCrossLine(int x1, int x2, int b1, int b2, double factor) {
		int [] crossline = new int[4];

		int x_dest = (int) (((b1-x1)*factor)+x1);
		int y_dest = (int) (((b2-x2)*factor)+x2);

		crossline[0] = (int) ((x1+x2-y_dest));
		crossline[1] = (int) ((x2+x_dest-x1));
		crossline[2] = crossline[0]+(x1-crossline[0])*2;
		crossline[3] = crossline[1]+(x2-crossline[1])*2;
		return crossline;
	} 
	
}
