package ics.uci.edu.Calico.Structures;

import ics.uci.edu.Calico.UI.DragHistory;
import ics.uci.edu.Calico.UI.ScrapPainter;
import ics.uci.edu.Calico.models.ScrapModel;

import java.awt.Color;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.FlatteningPathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import edu.umd.cs.piccolo.PNode;


public class ScrapController {

	public static boolean intersects(ScrapModel scrap, Rectangle2D bounds) {
		return scrap.encasingPoly.intersects(bounds);
	}
	
	public static boolean intersects(Polygon scrap, Polygon p) {
		return DragHistory.intersect(scrap, p);
	}
	
	public static boolean containsPoint(ScrapModel scrap, Point2D p) {
		return scrap.encasingPoly.contains(p);
	}

	public static void translate(ScrapModel scrap, int deltaX, int deltaY) {
		scrap.encasingPoly.translate(deltaX, deltaY);
		for (Polygon df : scrap.encasedFigures) {
			df.translate(deltaX, deltaY);			
		}
	}
	
	/**
	 * Translates ScrapModel scrap so that it's midpoint lies at Point p.
	 * @param scrap
	 * @param p
	 */
	public static void translateTo(ScrapModel scrap, Point2D p) {
		Point2D midPoint = getMidPoint(scrap);
		int deltaX = new Double(p.getX() - midPoint.getX()).intValue();
		int deltaY = new Double(p.getY() - midPoint.getY()).intValue();
		translate(scrap, deltaX, deltaY);
	}
	
	public static void addDrawingFigure(ScrapModel scrap, Polygon p, Color c) {
		scrap.encasedFigures.add(p);
		scrap.encasedFigureColors.add(c);
	}
	
	public static boolean intersectsFigure(ScrapModel scrap, Rectangle2D eraserBounds) {
		for (int i = 0; i < scrap.encasedFigures.size(); i++) {
			if (scrap.encasedFigures.get(i).intersects(eraserBounds) ||
					containedInBounds(eraserBounds, scrap.encasedFigures.get(i)))
				return true;
		}
		return false;
	}
	
	
	private static boolean containedInBounds(Rectangle2D eraserBounds,
			Polygon polygon) {
		for (int i = 0; i < polygon.npoints; i++)
			if (eraserBounds.contains(polygon.xpoints[i], polygon.ypoints[i]))
				return true;
		return false;
	}

	public static int getId(ScrapModel m) {
		return m.id;
	}

	public static void extend(ScrapModel scrap, Polygon polygon) {
		Area first, second;
		first = new Area(scrap.encasingPoly);
		second = new Area(polygon);
		
		first.add(second);
		
		scrap.encasingPoly = getPolyFromArea(first);
		
	}
	
	public static Polygon getPolyFromArea(Area a) {
		FlatteningPathIterator flattenedPath;
		flattenedPath = new FlatteningPathIterator(a.getPathIterator(null), 500);
		double[] coords = new double[6];

		Polygon temp = new Polygon();
		while (!flattenedPath.isDone()) {
			flattenedPath.currentSegment(coords);
			temp.addPoint((int)coords[0], (int)coords[1]);
			flattenedPath.next();
		}
		Polygon result = new Polygon();
		for (int i = 0; i < temp.npoints-1; i++)
			result.addPoint(temp.xpoints[i], temp.ypoints[i]);
		return result;
	}	
	
	public static ScrapPainter getScrapPainter(ScrapModel scrap, PNode parent) {
		for (int i = 0; i < parent.getChildrenCount(); i++) {
			if (parent.getChild(i) instanceof ScrapPainter &&
					((ScrapPainter)parent.getChild(i)).getModel().equals(scrap))
				return (ScrapPainter)parent.getChild(i);
		}
		
		return null;
	}
	
	public static ScrapModel copy(ScrapModel s) {
		ScrapModel ret = new ScrapModel(new Polygon(s.encasingPoly.xpoints, s.encasingPoly.ypoints, s.encasingPoly.npoints), s.id);
		for (Polygon df : s.encasedFigures) {
			ret.encasedFigures.add(new Polygon(df.xpoints, df.ypoints, df.npoints));
		}
		for (Color dfc : s.encasedFigureColors) {
			ret.encasedFigureColors.add(new Color(dfc.getRGB()));
		}
		ret.setBackgroundColor( new Color(s.getBackgroundColor().getRGB()) );
		ret.transparent = s.transparent;
		return ret;
	}
	
	public static Point2D getMidPoint(ScrapModel s) {
		int sumX = 0, sumY = 0;
		for (int i = 0; i < s.encasingPoly.npoints; i++) {
			sumX += s.encasingPoly.xpoints[i];
			sumY += s.encasingPoly.ypoints[i];
		}
		return new Point2D.Double(sumX/s.encasingPoly.npoints, sumY/s.encasingPoly.npoints);
	}
	
	public static void transformToRect(ScrapModel s) {
		Rectangle bounds = s.encasingPoly.getBounds();
		s.encasingPoly.reset();
		s.encasingPoly.addPoint(new Double(bounds.getX()).intValue(), new Double(bounds.getY()).intValue());
		s.encasingPoly.addPoint(new Double(bounds.getX() + bounds.getWidth()).intValue(), new Double(bounds.getY()).intValue());
		s.encasingPoly.addPoint(new Double(bounds.getX() + bounds.getWidth()).intValue(), new Double(bounds.getY() + bounds.getHeight()).intValue());
		s.encasingPoly.addPoint(new Double(bounds.getX()).intValue(), new Double(bounds.getY() + bounds.getHeight()).intValue());
	}
	
}
