package ics.uci.edu.Calico.UI;

import java.awt.Color;
import java.awt.Image;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Random;

import ics.uci.edu.Calico.Structures.BGElementController;
import ics.uci.edu.Calico.Structures.RelationshipController;
import ics.uci.edu.Calico.Structures.ScrapController;
import ics.uci.edu.Calico.UI.components.CalicoComponent;
import ics.uci.edu.Calico.UI.CalicoOptions;
import ics.uci.edu.Calico.models.BGElementModel;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.RelationshipModel;
import ics.uci.edu.Calico.models.ScrapModel;
import ics.uci.edu.Calico.models.SetScrapTransparency;
import ics.uci.edu.Calico.models.actions.*;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.components.CalicoGrid;
import ics.uci.edu.Calico.models.Snapshot;
import ics.uci.edu.Calico.models.actions.ChangedCell;
import ics.uci.edu.Calico.models.actions.CalicoAction;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;
import ics.uci.edu.Calico.models.actions.PenColorChanged;
import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolox.nodes.PLine;

import java.io.Serializable;

public class CalicoCanvas extends PCanvas implements Serializable{

	CalicoAction latestAction;
	
	//used to identify objects on the canvas. This number changes when the model is changed.
	private int objectIdTicker;
	private Color penColor;
	private Color bgColor;

	private ArrayList<CalicoActionListener> actionListeners;
	private ArrayList<CalicoComponent> components;
	
	
	//Used for the shared grid;
	public int sharedCanvasID;
	//ID remains static after program initiates. Used to identify
	//existing CalcioCanvases from each other (if communicating across networks
	//for example).
	public double cId;
	
	public CalicoCanvas() {
		objectIdTicker = 0;
		actionListeners = new ArrayList<CalicoActionListener>();
		components = new ArrayList<CalicoComponent>();
		
		bgColor = CalicoOptions.calico_bgcolor;
		setBackground(bgColor);
		penColor = Color.black;
		Random rand = new Random();
		cId = rand.nextDouble();
	}
	
	public void addComponent(CalicoComponent cc) {
		components.add(cc);
		//Note, cc must set its own bounds.
		getCamera().addChild(cc);
		cc.registerCanvas(this);
	}
	
	//added by mrbrad
	public void removeAllComponents(){
		for (CalicoComponent cc : components) {
			getLayer().removeChild(cc);
		}
	}
	
	public ArrayList<CalicoComponent> getCalicoComponents() {
		return components;
	}
	
	public void setBounds(Rectangle r) {
		setBounds(r.x, r.y, r.width, r.height);
	}
	
	public void setBounds(int x, int y, int w, int h) {
		//remove EmptyBoundingBox if it exists.
		setBoundsMarkers(w, h);
		
		for (CalicoComponent cc : components)
			cc.updateBounds(w, h);
		super.setBounds(x, y, w, h);
	}
	
	public void addCalicoActionListener(CalicoActionListener listener) {
		System.out.println("action listener added");
		actionListeners.add(listener);
	}
	
	public boolean removeCalicoActionListener(CalicoActionListener listener) {
		System.out.println("action listener removed");
		return actionListeners.remove(listener);
	}
	
	public ArrayList<CalicoActionListener> getListeners() {
		return actionListeners;
	}	
	
	public ObjectHandlerModel getModel() {
		ObjectHandlerModel m = new ObjectHandlerModel();
		
		//populate it
		PNode node;
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			node = getLayer().getChild(i);
			if (node instanceof ScrapPainter)
				m.scraps.add(ScrapController.copy(((ScrapPainter)node).getModel()));
			else if (node instanceof RelationshipPainter)
				m.relationships.add(RelationshipController.copy(((RelationshipPainter)node).getModel()));
			else if (node instanceof BGElementPainter &&
					((BGElementPainter)node).getId() > -1)
				m.bgElements.add(BGElementController.copy(((BGElementPainter)node).getModel()));
		}
		m.objectIdTicker = objectIdTicker;
		m.bgColor = getBGColor();
		
		return m;
	}
	
	@Deprecated
	public void changeModel(ObjectHandlerModel model) {
		loadModel(model);
	}
	
	public void loadModel(ObjectHandlerModel model) {
		getLayer().removeAllChildren();
		
		setBGColor(model.bgColor);
		ScrapPainter scrapPainter;
		for (ScrapModel s : model.scraps) {
			scrapPainter = new ScrapPainter(s);			
			getLayer().addChild(scrapPainter);
			scrapPainter.setBounds();
		}
		RelationshipPainter relPainter;
		for (RelationshipModel r : model.relationships) {
			relPainter = new RelationshipPainter(r);
			relPainter.setBounds();
			getLayer().addChild(relPainter);
			relPainter.moveToFront();
		}
		BGElementPainter bgePainter;
		for (BGElementModel b : model.bgElements) {
			bgePainter = new BGElementPainter(b);
			bgePainter.setBounds(bgePainter.getPoly().getBounds2D());
			getLayer().addChild(bgePainter);
			bgePainter.moveToBack();
		}
		setBoundsMarkers(getBounds().width, getBounds().height);
		
		objectIdTicker = model.objectIdTicker;
	}
	
	public boolean applyAction(CalicoAction action) {
		boolean ret = false;
		
		System.out.println("CalicoCanvas " + cId + " is performing " + action.getAction());
		
		if (action instanceof ScrapsRaised) {
			raiseScraps((ScrapsRaised)action);
		}
		else if (action instanceof AddInternalScrap) {
			addInternalScrap((AddInternalScrap)action);
			ret = true;
		}
		else if (action instanceof AddRelationship) {
			addRelationship((AddRelationship)action);
			ret = true;			
		}
		else if (action instanceof AddScrap) {
			addScrap((AddScrap)action);
			ret = true;
		}
		else if (action instanceof DeleteScraps) {
			deleteScraps((DeleteScraps)action);
			ret = true;
		}
		else if (action instanceof DeselectScraps) {
			
		}
		else if (action instanceof ExtendScrap) {
			extendScrap((ExtendScrap)action);
			ret = true;
		}
		else if (action instanceof LoadState) {
			
		}
		else if (action instanceof SelectScraps) {
			selectScraps((SelectScraps)action);
			ret = true;
		}
		else if (action instanceof ThinRelationship) {
			thinRelationship((ThinRelationship)action);
		}
		else if (action instanceof ThickenRelationship) {
			thickenRelationship((ThickenRelationship)action);
		}
		else if (action instanceof DeleteRelationship) {
			deleteRelationship((DeleteRelationship)action);
		}
		else if (action instanceof TranslateScrap) {
			translateScrap((TranslateScrap)action);
		}
		else if (action instanceof AddPremadeScrap) {
			addPremadeScrap((AddPremadeScrap)action);
		}
		else if (action instanceof CreateBackgroundElement) {
			createBGElem((CreateBackgroundElement)action);
		} 
		else if (action instanceof LiftScrapFromBackground) {
			liftScrapFromBG((LiftScrapFromBackground)action);
		}
		else if (action instanceof DropScrapToBackground) {
			dropScraptoBG((DropScrapToBackground)action);
		}
		else if (action instanceof DuplicateScrap) {
			duplicateScrap((DuplicateScrap)action);
		}
		else if (action instanceof BGEErase) {
			eraseBGElement((BGEErase)action);
		}
		else if (action instanceof SFErase) {
			eraseSFElement((SFErase)action);
		}
		else if (action instanceof UndoAction) {
			undoAction((UndoAction)action);
		}
		else if (action instanceof TransformScrapToRect) {
			transformScrapToRect((TransformScrapToRect)action);
		}
		else if (action instanceof PenColorChanged) {
			penColorChanged((PenColorChanged)action);
		}
		else if (action instanceof SetScrapTransparency) {
			setScrapTransparency((SetScrapTransparency)action);
		}
		else {
			//System.err.println("PrimaryFrame:applyAction() improper action passed: " + action);
		}
		
		latestAction = action;
		fireListeners();
		repaint();
		
		//printContents();
		
		return ret;
	}
	
	private void setScrapTransparency(SetScrapTransparency action) {
		ScrapPainter painter = getScrap(action.scrapId);
		painter.setTransparency(action.transparency);		
	}

	private void penColorChanged(PenColorChanged action) {
		setPenColor(action.color);	
	}

		
	private void raiseScraps(ScrapsRaised action) {
		for (int i = 0; i < action.scraps.length; i++) {
			moveToFront(getScrap(action.scraps[i]), null);
		}
		
	}

	private void eraseBGElement(BGEErase action) {
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			if (getLayer().getChild(i) instanceof BGElementPainter &&
					((BGElementPainter)getLayer().getChild(i)).getId() == action.modelId) {
				trim((BGElementPainter)getLayer().getChild(i), action.getRect());
				return;
			}
				
		}
		
	}
	
	private void eraseSFElement(SFErase action) {
		trim(getScrap(action.scrapId), action.getRect());
	}

	private void undoAction(UndoAction action) {
		loadModel(action.ohm);
	}

	private void translateScrap(TranslateScrap action) {
		ScrapPainter painter, oPainter;
		ScrapModel model;
		
		for (int i = 0; i < action.scrapId.length; i++) {
			painter = getScrap(action.scrapId[i]);
			painter.setTranslucent(false);
			model = painter.getModel();
			painter.translateTo(new Point2D.Double(action.finalX[i], action.finalY[i]));
			painter.setBounds();
		}
		
	}

	private void transformScrapToRect(TransformScrapToRect action) {
		ScrapPainter scrap = getScrap(action.scrapId);
		ScrapController.transformToRect(scrap.getModel());
		scrap.setWidget();
		scrap.setBounds();		
	}

	private void duplicateScrap(DuplicateScrap action) {
		ScrapPainter original = getScrap(action.scrapId);
		ScrapPainter dup = new ScrapPainter(ScrapController.copy(original.getModel()));
		dup.getModel().id = this.objectIdTicker++;
		getLayer().addChild(dup);
		dup.translate(10, 10);
		dup.setWidget();
				
	}

	private void dropScraptoBG(DropScrapToBackground action) {
		
		ScrapPainter painter = getScrap(action.scrapId);
		ScrapModel model = painter.getModel();
		Polygon bge;
		boolean dropped;
		
		BGElementPainter bgePainter;
		for (int i = 0; i < model.encasedFigures.size(); i++) {
			dropped = false;
			bgePainter = new BGElementPainter(
					new BGElementModel(model.encasedFigures.get(i), model.encasedFigureColors.get(i), objectIdTicker++));
			//check if there's another scrap to drop it on to
			for (int j = getLayer().getChildrenCount() - 1; j >= 0; j--) {
				
				bge = bgePainter.getPoly();
				if (getLayer().getChild(j) instanceof ScrapPainter &&
						((ScrapPainter)getLayer().getChild(j)).contains(bge) && 
						((ScrapPainter)getLayer().getChild(j)).getModel().id != action.scrapId) {
					//drop it on to that scrap
					((ScrapPainter)getLayer().getChild(j)).getModel().encasedFigures.add(bge);
					((ScrapPainter)getLayer().getChild(j)).getModel().encasedFigureColors.add(bgePainter.getColor());
					//((ScrapPainter)getLayer().getChild(j)).repaint();
					dropped = true;
				}
			}
			if (!dropped) {
				bgePainter.setBounds(bgePainter.getPoly().getBounds2D());
				getLayer().addChild(0, bgePainter);
			}
			//getLayer().addChild(0, bgePainter);
		}
		
		deleteScraps(new DeleteScraps(new int[]{action.scrapId}));		
	}

	private void liftScrapFromBG(LiftScrapFromBackground action) {
		// TODO Auto-generated method stub
		
	}

	private void createBGElem(CreateBackgroundElement action) {
		BGElementModel bge = new BGElementModel(action.poly, penColor, objectIdTicker++);
		BGElementPainter painter = new BGElementPainter(bge);
		painter.setBounds(painter.getPoly().getBounds2D());
		getLayer().addChild(0, painter);
	}
	
	private void trim(BGElementPainter bge, Rectangle2D bounds) {
		BGElementModel model = bge.getModel();
		
		ArrayList<Polygon> trimmedPoly = this.polyTrim(model.poly, bounds);
		
		if (trimmedPoly.size() == 0) {
			bge.clear();
			bge.getParent().removeChild(bge);
		}
		else if (trimmedPoly.size() == 1) {
			bge.setPoly(trimmedPoly.get(0));
		}
		else if (trimmedPoly.size() == 2) {
			//low model
			getLayer().removeChild(bge);
			
			//low model
			BGElementModel lowModel = new BGElementModel(trimmedPoly.get(0), model.color, objectIdTicker++);
			BGElementPainter lowPainter = new BGElementPainter(lowModel);
			lowPainter.setBounds(lowModel.poly.getBounds2D());
			getLayer().addChild(lowPainter);
			lowPainter.moveToBack();
			
			//high model
			BGElementModel highModel = new BGElementModel(trimmedPoly.get(1), model.color, objectIdTicker++);
			BGElementPainter highPainter = new BGElementPainter(highModel);
			highPainter.setBounds(highModel.poly.getBounds2D());
			getLayer().addChild(highPainter);
			highPainter.moveToBack();
		}
		else {
			System.err.println("Something wrong in CalicoCanvas:trim");
		}	
	}
	
	private void trim(ScrapPainter sp, Rectangle2D bounds) {
		ArrayList<Polygon> tempPolies;
		ArrayList<Integer> toBeRemoved = new ArrayList<Integer>();
		
		for (int i = 0; i < sp.getModel().encasedFigures.size(); i++) {
			tempPolies = polyTrim(sp.getModel().encasedFigures.get(i), bounds);
			if (tempPolies.size() == 0)
				toBeRemoved.add(new Integer(i));
			else if (tempPolies.size() == 2) {
				sp.getModel().encasedFigures.add(tempPolies.get(0));
				sp.getModel().encasedFigureColors.add(sp.getModel().encasedFigureColors.get(i));
				sp.getModel().encasedFigures.add(tempPolies.get(1));
				sp.getModel().encasedFigureColors.add(sp.getModel().encasedFigureColors.get(i));
				toBeRemoved.add(new Integer(i));
			}
			else if (tempPolies.size() == 1) {
				sp.getModel().encasedFigures.set(i, tempPolies.get(0));
			}
			else {
				System.err.println("Something wrong in CalicoCanvas:trim");
			}
		}
		for (int i = toBeRemoved.size()-1; i > -1; i--) {
			sp.getModel().encasedFigures.remove(toBeRemoved.get(i).intValue());
			sp.getModel().encasedFigureColors.remove(toBeRemoved.get(i).intValue());
		}
		
	}
	
	private ArrayList<Polygon> polyTrim(Polygon p, Rectangle2D bounds) {
		ArrayList<Polygon> ret = new ArrayList<Polygon>();
		
		Line2D line;
		int intersectionLow = -1, intersectionHigh = -1;
		for (int i = 1; i < p.npoints; i++) {
			line = new Line2D.Double(p.xpoints[i-1], p.ypoints[i-1],
					p.xpoints[i], p.ypoints[i]);
			if (bounds.intersectsLine(line)) {
				if (intersectionLow == -1)
					intersectionLow = i -1;
				intersectionHigh = i;
			}
		}
		
		if (intersectionLow > 1) {
			Polygon low = new Polygon();
			for (int i = 0; i < intersectionLow; i++) {
				low.addPoint(p.xpoints[i], p.ypoints[i]);
			}
			ret.add(low);
		}
		if (intersectionHigh > p.npoints - 3) {
		}
		else {
			Polygon highP = new Polygon();
			for (int i = intersectionHigh+1; i < p.npoints; i++) {
				highP.addPoint(p.xpoints[i], p.ypoints[i]);
			}
			ret.add(highP);
		}
		
		return ret;
	}

	private void fireListeners() {
		for (CalicoActionListener listener : actionListeners) {
			listener.fireCalicoListener(new CalicoActionEvent(latestAction, this, cId));
		}
	}
	
	public void fireListeners(CalicoAction outsideAction) {
		for (CalicoActionListener listener : actionListeners) {
			listener.fireCalicoListener(new CalicoActionEvent(outsideAction, this, cId));
		}
	}

	private void deleteRelationship(DeleteRelationship action) {
		PNode node = null;
		for (int j = 0; j < action.relId.length; j++) {
			for (int i = 0; i < getLayer().getChildrenCount(); i++) {
				node = getLayer().getChild(i);
				if (node instanceof RelationshipPainter &&
						((RelationshipPainter)node).getModel().equals(action.relId[j])) {
					getLayer().removeChild(node);
				}
			}
		}
	}

	private void thickenRelationship(ThickenRelationship action) {

		PNode node = null;
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			node = getLayer().getChild(i);
			if (node instanceof RelationshipPainter &&
					((RelationshipPainter)node).getModel().equals(action.relId)) {
				((RelationshipPainter)node).getModel().thickness += action.amount;
			}
		}
		if (node != null)
			node.repaint();
		
	}

	private void thinRelationship(ThinRelationship action) {
		PNode node = null;
		RelationshipPainter painter;
		
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			node = getLayer().getChild(i);
			if (node instanceof RelationshipPainter &&
					((RelationshipPainter)node).getModel().equals(action.relId)) {
				painter = (RelationshipPainter)node;
				painter.getModel().thickness -= action.amount;
				if (painter.getModel().thickness < 1)
					painter.getModel().thickness = 1;
			}
		}
		if (node != null)
			node.repaint();
		
	}

	private void extendScrap(ExtendScrap action) {
		ScrapPainter painter = getScrap(action.scrapId);
		ScrapController.extend(painter.getModel(), action.extension);
		painter.setWidget();
		painter.setBounds();
	}

	private void addRelationship(AddRelationship action) {
		RelationshipModel m = new RelationshipModel(action.scrapId1, action.scrapId2, 
				action.point1X, action.point1Y, action.point2X, action.point2Y, objectIdTicker++);
		m.setColor(getPenColor());
		RelationshipPainter painter = 
			new RelationshipPainter(m);
		painter.setBounds();
		getLayer().addChild(painter);
	}

	private void addInternalScrap(AddInternalScrap action) {
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			if (getLayer().getChild(i) instanceof ScrapPainter &&
					((ScrapPainter)getLayer().getChild(i)).getModel().equals(action.scrapId)) {
				ScrapController.addDrawingFigure(
						((ScrapPainter)getLayer().getChild(i)).getModel(), 
						action.p, penColor);
			}
		}
		
	}

	private void selectScraps(SelectScraps scraps) {
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			if (getLayer().getChild(i) instanceof ScrapPainter)
				for (int j = 0; j < scraps.scrapIds.length; j++)
					if (((ScrapPainter)getLayer().getChild(i)).getModel().equals(scraps.scrapIds[j]))
						((ScrapPainter)getLayer().getChild(i)).select();
		}
		
	}

	private void addScrap(AddScrap action) {
		//get the highest BGElement		
		ScrapModel temp = new ScrapModel(action.p, objectIdTicker++);
		
		BGElementModel bgeModel;
		for (int i = getLayer().getChildrenCount() - 1; i >= 0; i--) {
			if (getLayer().getChild(i) instanceof BGElementPainter) {
				bgeModel = ((BGElementPainter)getLayer().getChild(i)).getModel();
				if (BGElementController.containedBy(bgeModel, action.p)) {
					getLayer().removeChild(i);
					ScrapController.addDrawingFigure(
							temp, 
							bgeModel.poly, bgeModel.color);
				}
			}
		}
		
		ScrapPainter painter = new ScrapPainter(temp);
		painter.setBounds();
		
		//get the point of the highest BGElement
		int highestBGElem = -1;
		for (int i = 0; i < getLayer().getChildrenCount(); i++)
			if (getLayer().getChild(i) instanceof BGElementPainter &&
				((BGElementPainter)getLayer().getChild(i)).getId() > -1)
				if (i > highestBGElem)
					highestBGElem = i;
		
		getLayer().addChild(highestBGElem+1, painter);
	}
	
	/**
	 * Like add scrap, we just need to set the id of the scrap
	 * @param scrap
	 */
	private void addPremadeScrap(AddPremadeScrap action) {
		
		RelationshipPainter painterR;
		for (RelationshipModel rm : action.rels) {
			rm.id = objectIdTicker++;
			painterR = new RelationshipPainter(rm);
			painterR.setBounds();
			getLayer().addChild(painterR);
		}
		
		ScrapPainter painterS;
		int oldId;
		for (ScrapModel sm : action.scraps) {
			oldId = sm.id;
			sm.id = objectIdTicker++;
			painterS = new ScrapPainter(sm);
			painterS.setBounds();
			getLayer().addChild(painterS);
			
			//gotta update relationship models for changed hooks
			for (RelationshipModel rm : action.rels) {
				if (rm.hook1 == oldId)
					rm.hook1 = sm.id;
				else if (rm.hook2 == oldId)
					rm.hook2 = sm.id;
			}
		}
		
		
		
	}
	
	/**
	 * Removes all of the scraps that contain the included ID from the canvas. Scraps should only exist
	 * in the layer returned by getLayer()
	 * @param action
	 */
	private void deleteScraps(DeleteScraps action) {
		//delete the scraps contained in action
		for (int i = getLayer().getChildrenCount() - 1; i >= 0 ; i--) {
			if (getLayer().getChild(i) instanceof ScrapPainter)
				for (int j = 0; j < action.scrapIds.length; j++)
					if (((ScrapPainter)getLayer().getChild(i)).getModel().equals(action.scrapIds[j])) {
						getLayer().removeChild(i);
						break;
					}
		}
		
		RelationshipPainter rPainter;
		//remove the relationships attached to the deleted scraps
		for (int i = getLayer().getChildrenCount() - 1; i >= 0; i--) {
			if (getLayer().getChild(i) instanceof RelationshipPainter) {
				rPainter = (RelationshipPainter)getLayer().getChild(i);
				for (int j = 0; j < action.scrapIds.length; j++) 
					if (RelationshipController.connectedTo(
							(rPainter).getModel(), action.scrapIds[j])) {
						getLayer().removeChild(i);
						break;
					}
				
			}
		}
	}
	

	public ScrapPainter getScrap(int id) {
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			if (getLayer().getChild(i) instanceof ScrapPainter &&
					((ScrapPainter)getLayer().getChild(i)).getModel().equals(id))
				return (ScrapPainter)getLayer().getChild(i);
		}
		return null;
	}
	
	
	public Image getImage() {
		return getImage(new Color(0,0,0,0));
	}
	
	public Image getImage(Color background) {
		//if (getLayer().getBounds().width < 1 || getLayer().getBounds().height < 1) {
		if (getBounds().width < 1 || getBounds().height < 1)
			this.setBounds(0, 0, 500, 500);
		BufferedImage image = new BufferedImage(getBounds().width, getBounds().height, BufferedImage.TYPE_INT_ARGB);
		
		return getLayer().toImage(image, background);
	}
	
	public BufferedImage getBufferedImage(Color background) {
		if (getBounds().width < 1 || getBounds().height < 1)
			this.setBounds(0, 0, 500, 500);
		BufferedImage image = new BufferedImage(getBounds().width, getBounds().height, BufferedImage.TYPE_INT_ARGB);
		
		return (BufferedImage)getLayer().toImage(image, background);
	}
	
	public static Image getProportionateScaledImage(Image img, int width, int height, int hints) {
		//figure out ratio
		double ratio;
		double imgW = img.getWidth(null);
		double imgH = img.getHeight(null);

		if (width/imgW < height/imgH)
			ratio = width/imgW;
		else
			ratio = height/imgH;
		
		int finalWidth = new Double(img.getWidth(null)*ratio).intValue();
		int finalHeight = new Double(img.getHeight(null)*ratio).intValue();
		
		return img.getScaledInstance(finalWidth, finalHeight, hints);
	}
	
	public Image getImage(int boundsX, int boundsY) {
		if (getLayer().getChildrenCount() == 0) {
			BufferedImage image = new BufferedImage(100, 100, BufferedImage.TYPE_INT_ARGB);
			//image.getGraphics().setColor(new Color(0,0,0,0));
			//image.getGraphics().fillRect(0, 0, 100, 100);
			return image;
		}
		
		
		else 
			return getLayer().toImage(boundsX, boundsY, null);
	}
	
	public Image getNodeImage(int nodeId) {
		ScrapPainter scrap = getScrap(nodeId);
		return scrap.toImage();
	}

	public int getObjectIdTicker(){
		return objectIdTicker;
	}
	
	public ArrayList<ScrapPainter> getSelectedScraps() {
		ArrayList<ScrapPainter> ret = new ArrayList<ScrapPainter>();
		
		for (Object node : getLayer().getAllNodes()) {
			if (node instanceof ScrapPainter && ((ScrapPainter)node).isSelected())
				ret.add((ScrapPainter)node);
		}
		return ret;
	}
	
	public void moveToFront(ScrapPainter painter, ArrayList<PNode> processed) {
		boolean isTop = false;
		if (processed == null) {
			isTop = true;
			processed = new ArrayList<PNode>();
		}
		
		if (processed.contains(painter))
			return;
		processed.add(painter);
		ArrayList<ScrapPainter> occluded = getOccluding(painter);
		painter.moveToFront();
		
		//move attached relationships to front
		/*
		ArrayList<RelationshipPainter> attachedRels = getAttachedRels(painter);
		for (RelationshipPainter rp : attachedRels) {
			if (rp.getModel().hook1 == painter.getModel().id ||
					rp.getModel().hook2 == painter.getModel().id) {
				processed.add(rp);
				rp.moveToFront();
			}
		}*/
		
		//move occluded items to the front
		for (ScrapPainter sp : occluded)
			moveToFront(sp, processed);
		
		if (isTop) {
			ArrayList<RelationshipPainter> rels = new ArrayList<RelationshipPainter>();
			for (int i = 0; i < getLayer().getChildrenCount(); i++)
				if (getLayer().getChild(i) instanceof RelationshipPainter)
					rels.add((RelationshipPainter)getLayer().getChild(i));
			for (RelationshipPainter rp : rels)
				rp.moveToFront();
		}
		
	}
	
	private ArrayList<RelationshipPainter> getAttachedRels(ScrapPainter sp) {
		ArrayList<RelationshipPainter> ret = new ArrayList<RelationshipPainter>();
		
		for (Object node : getLayer().getAllNodes()) {
			if (node instanceof RelationshipPainter && 
					RelationshipController.connectedTo(((RelationshipPainter)node).getModel(), sp.getModel().id))
				ret.add((RelationshipPainter)node);
		}
		
		return ret;
	}
	
	public ArrayList<ScrapPainter> getOccluding(ScrapPainter sp) {
		PNode parent = sp.getParent();
		ArrayList<ScrapPainter> ret = new ArrayList<ScrapPainter>();
		
		for (int i = 0; i < parent.getChildrenCount(); i++) {
			if (parent.getChild(i) instanceof ScrapPainter &&
					((ScrapPainter)parent.getChild(i)).intersects(sp.getModel().encasingPoly)) {
				if (!((ScrapPainter)parent.getChild(i)).getModel().equals(sp.getModel()) &&
						((ScrapPainter)parent.getChild(i)).getDrawingPriority() > sp.getDrawingPriority())
					ret.add((ScrapPainter)parent.getChild(i));
			}
		}
		
		return ret;
	}
	
	private void setBoundsMarkers(int w, int h) {
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			if (getLayer().getChild(i) instanceof BoundsMarker)
				getLayer().removeChild(i);
		}
		BoundsMarker point = new BoundsMarker();
		point.addPoint(0, w, h);
		point.addPoint(1, w, h);
		getLayer().addChild(point);
		BoundsMarker point2 = new BoundsMarker();
		point2.addPoint(0, 0, 0);
		point2.addPoint(1, 0, 0);
		getLayer().addChild(point2);
	}
	
	public void setSharedCanvasID(int i){
		sharedCanvasID = i;
	}
	
	
	
	/**
	 * This class is an empty placeholder that guarantees that the layer that holds it has
	 * a bounds that's at least the size of this node. This is to get around the peculiar 
	 * image properties of PNode.toImage.
	 * @author calvmari
	 *
	 */
	private class BoundsMarker extends PLine {
		
	}

	public void setPenColor(Color c) {
		penColor = c;
		
	}
	
	public Color getPenColor() {
		return penColor;
	}
	
	public void setBGColor(Color c) {
		bgColor = c;
		setBackground(c);
	}
	
	public Color getBGColor() {
		return bgColor;
	}
	
	public void printContents() {
		for (int i = 0; i < getLayer().getChildrenCount(); i++) {
			System.out.print("Layer Child( ' " + i + "') = " + getLayer().getChild(i).getClass().toString());
			if (getLayer().getChild(i) instanceof ScrapPainter)
				System.out.print(" id: " + ((ScrapPainter)getLayer().getChild(i)).getModel().id);
			if (getLayer().getChild(i) instanceof BGElementPainter)
				System.out.print(" id: " + ((BGElementPainter)getLayer().getChild(i)).getModel().id);
			System.out.println();
		}
	}
	
	public CalicoAction getLastAction(){
		return latestAction;
	}
	
	public void clearInvalidNodes() {
		for (int i = getLayer().getChildrenCount() - 1; i > -1; i--) {
			if (getLayer().getChild(i) instanceof BGElementPainter &&
					((BGElementPainter)getLayer().getChild(i)).getId() < 0)
				getLayer().removeChild(i);
		}
	}
	
}


