package ics.uci.edu.Calico.UI;

import java.util.regex.*;

import java.lang.reflect.*;

import java.io.*;
import java.util.Scanner;
import java.awt.Color;
import java.util.HashMap;
import java.util.ArrayList;

import org.json.me.*;

/*
Config file should be in the same folder that you are running Calico from.

Call it "config.cfg"

Syntax:
	<var_name> <value>

Example:
	pal_rows 5
	
you can use "//" as comments.

*/

public class CalicoOptions
{
	public static boolean debug = true;
	
	// Palettes
	public static int pal_rows = 5;
	public static int pal_cols = 2;
	public static String pal_server = "127.0.0.1";
	public static int pal_port = 8787;
	public static int pal_timeout = 1000;
	public static String pal_savetype = "FILE"; // SERVER, FILE, BOTH?
	
	// Grid
	public static int grid_rows = 14;
	public static int grid_cols = 14;
	
	// Global Calico Stuff
	public static boolean calico_fullscreen = true;
	public static int calico_screenwidth = 1024;
	public static int calico_screenheight = 768;
	public static String calico_savepath = "./";
	public static ArrayList<Integer> calico_zoomlevels = new ArrayList<Integer>();
	
	// Calico Color Stuff
	public static HashMap<String,Color> calico_bgcolors = new HashMap<String,Color>();
	public static ArrayList<Color> calico_pencolors = new ArrayList<Color>();
	
	public static Color calico_bgcolor = Color.WHITE;
	public static Color calico_default_pen_color = Color.BLACK;
	
	
	public static ArrayList<String> calico_test_str_arr = new ArrayList<String>();
	public static ArrayList<Double> calico_test_dbl_arr = new ArrayList<Double>();

	public static ArrayList<String> modules_active = new ArrayList<String>();

	// Networking
	public static int network_port = 6112;
	
	public static void debug(String str)
	{
		if(debug)
		{
			System.out.println("DEBUG: " + str);
		}
	}
	
	// 

	public static void setDefaults()
	{
		calico_pencolors.add(Color.BLACK);
		calico_pencolors.add(Color.GREEN);
		calico_pencolors.add(Color.RED);
		calico_pencolors.add(Color.BLUE);
		
		modules_active.add("PALETTES");
		modules_active.add("PENCOLOR");
		modules_active.add("ZOOM");
		modules_active.add("PAN");
		
		
		/*calico_bgcolors.add(Color.WHITE);
		calico_bgcolors.add(Color.WHITE);
		calico_bgcolors.add(Color.WHITE);
		calico_bgcolors.add(Color.WHITE);
		*/
	}

	
	
	public static void readConfig(String aFileName)
	{
		setDefaults();
		try
		{
			File fFile = new File(aFileName);
			Scanner scanner = new Scanner(fFile);
			
			Pattern pattern = Pattern.compile("^([a-z_0-9]+) (.+)$");
			
			Class c = Class.forName("ics.uci.edu.Calico.UI.CalicoOptions");
			
			//first use a Scanner to get each line
			while ( scanner.hasNextLine() )
			{
				String curline = scanner.nextLine();
				Matcher matcher = pattern.matcher(curline);
				if(matcher.matches())
				{
					String key = matcher.group(1);
					String value = matcher.group(2);
					try
					{
						Field f = c.getField(key);
						
						String vartype = f.getType().toString();
						
						if(vartype.equals("class java.lang.String") )
						{
							f.set(null, value);
						}
						else if(vartype.equals("int"))
						{
							f.setInt(null, getIntValue(f.getInt(null), value) );
						}
						else if(vartype.equals("boolean"))
						{
							f.setBoolean(null, getBooleanValue(f.getBoolean(null), value) );
						}
						else if(vartype.equals("byte"))
						{
							f.setByte(null, getByteValue(f.getByte(null), value) );
						}
						else if(vartype.equals("long"))
						{
							f.setLong(null, getLongValue(f.getLong(null), value) );
						}
						else if(vartype.equals("double"))
						{
							f.setDouble(null, getDoubleValue(f.getDouble(null), value) );
						}
						else if(vartype.equals("float"))
						{
							f.setFloat(null, getFloatValue(f.getFloat(null), value) );
						}
						else if(vartype.equals("char"))
						{
							f.setChar(null, getCharValue(f.getChar(null), value) );
						}
						else if(vartype.equals("short"))
						{
							f.setShort(null, getShortValue(f.getShort(null), value) );
						}
						else if(vartype.equals("class java.awt.Color"))
						{
							f.set(null, getColorValue( (Color)f.get(null), value) );
						}
						else if(vartype.equals("class java.util.ArrayList"))
						{
							Type type = f.getGenericType();
							if (type instanceof ParameterizedType)
							{
								ParameterizedType pt = (ParameterizedType) type;
								Type[] types = pt.getActualTypeArguments();
								String typename = types[0].toString();
								if( typename.equals("class java.lang.Integer"))
								{
									f.set(null, getIntArray( (ArrayList<Integer>) f.get(null), value) );
								}
								else if(typename.equals("class java.lang.Double"))
								{
									f.set(null, getDoubleArray( (ArrayList<Double>) f.get(null), value) );
								}
								else if( typename.equals("class java.lang.String"))
								{
									f.set(null, getStringArray( (ArrayList<String>) f.get(null), value) );
								}
								else if( typename.equals("class java.awt.Color"))
								{
									f.set(null, getColorArray( (ArrayList<Color>) f.get(null), value) );
								}

							}
						}
						else if(vartype.equals("class java.util.HashMap"))
						{
							JSONObject temp = new JSONObject(value);
							//System.out.println("JSON: " + temp.names());
							Type type = f.getGenericType();
							if( type instanceof ParameterizedType)
							{
								ParameterizedType pt = (ParameterizedType) type;
								Type[] types = pt.getActualTypeArguments();
								String keytype = types[0].toString();
								String typename = types[1].toString();
								
								if( typename.equals("class java.lang.Integer"))
								{
									System.out.println("NOT IMPLEMENTED");
									//f.set(null, getIntArray( (ArrayList<Integer>) f.get(null), value) );
								}
								else if(typename.equals("class java.lang.Double"))
								{
									System.out.println("NOT IMPLEMENTED");
									//f.set(null, getDoubleArray( (ArrayList<Double>) f.get(null), value) );
								}
								else if( typename.equals("class java.lang.String"))
								{
									System.out.println("NOT IMPLEMENTED");
									//f.set(null, getStringArray( (ArrayList<String>) f.get(null), value) );
								}
								else if( typename.equals("class java.awt.Color"))
								{
									f.set(null, getColorHash( (HashMap<String,Color>) f.get(null), value) );
								}

							}
						}

						//getColorValue
						System.out.println("CONFIG: " + key + " = " + f.get(null));
						
					}
					catch(Exception e)
					{
						System.out.println("CONFIG: Unknown variable: "+key);
					}
					
				}//hasmatch
			}
			scanner.close();
		}
		catch(Exception e)
		{
			//ensure the underlying stream is always closed
			System.out.println(e);
		}
	}
	
	
	private static boolean getBooleanValue(boolean curVal, String newVal)
	{
		newVal = newVal.toUpperCase();
		if(newVal.equals("YES") || newVal.equals("TRUE") || newVal.equals("1") || newVal.equals("ON") )
		{
			return true;
		}
		else if(newVal.equals("NO") || newVal.equals("FALSE") || newVal.equals("0") || newVal.equals("OFF") )
		{
			return false;
		}
		else
		{
			return curVal;
		}
	}
	
	private static int getIntValue(int curVal, String newVal)
	{
		try { return Integer.parseInt(newVal); }
		catch(Exception e) { return curVal; }
	}
	
	private static String setString(String curVal, String newVal)
	{
		return newVal;
	}

	private static double getDoubleValue(double curVal, String newVal)
	{
		try { return Double.parseDouble(newVal); }
		catch(Exception e) { return curVal; }
	}
	
	private static short getShortValue(short curVal, String newVal)
	{
		try { return Short.parseShort(newVal); }
		catch(Exception e) { return curVal; }
	}

	private static long getLongValue(long curVal, String newVal)
	{
		try { return Long.parseLong(newVal); }
		catch(Exception e) { return curVal; }
	}
	
	private static byte getByteValue(byte curVal, String newVal)
	{
		try { return Byte.parseByte(newVal); }
		catch(Exception e) { return curVal; }
	}
	
	private static float getFloatValue(float curVal, String newVal)
	{
		try { return Float.parseFloat(newVal); }
		catch(Exception e) { return curVal; }
	}
	private static char getCharValue(char curVal, String newVal)
	{
		return newVal.charAt(0);
	}

	private static HashMap<String,Color> getColorHash(HashMap<String,Color> curval, String newval)
	{
		try
		{
			JSONObject temp = new JSONObject(newval);
			JSONArray names = temp.names();
			HashMap<String,Color> newlist = new HashMap<String,Color>();
			for(int i=0;i<names.length();i++)
			{
				Color nclr = getColorValue( null, temp.getString( (String)names.get(i) ) );
				if( nclr != null )
				{
					newlist.put( (String)names.get(i), nclr);
				}
			}
			return newlist;
		}
		catch(Exception e)
		{
			return curval;
		}
	}
	
	private static HashMap<String,String> getStringHash(HashMap<String,String> curval, String newval)
	{
		try
		{
			JSONObject temp = new JSONObject(newval);
			JSONArray names = temp.names();
			HashMap<String,String> newlist = new HashMap<String,String>();
			for(int i=0;i<names.length();i++)
			{
				newlist.put( (String)names.get(i), temp.getString((String)names.get(i)) );
			}
			return newlist;
		}
		catch(Exception e)
		{
			return curval;
		}
	}
	
	private static HashMap<String,Integer> getIntHash(HashMap<String,Integer> curval, String newval)
	{
		try
		{
			JSONObject temp = new JSONObject(newval);
			JSONArray names = temp.names();
			HashMap<String,Integer> newlist = new HashMap<String,Integer>();
			for(int i=0;i<names.length();i++)
			{
				newlist.put( (String)names.get(i), (Integer)temp.get((String)names.get(i)) );
			}
			return newlist;
		}
		catch(Exception e)
		{
			return curval;
		}
	}

	private static Color getColorByName(String color)
	{
		color = color.toUpperCase();

		HashMap<String, Color> colors = new HashMap<String,Color>();
		colors.put("BLACK",Color.BLACK);
		colors.put("BLUE",Color.BLUE);
		colors.put("CYAN",Color.CYAN);
		colors.put("GRAY",Color.GRAY);
		colors.put("RED",Color.RED);
		colors.put("WHITE",Color.WHITE);
		colors.put("YELLOW",Color.YELLOW);
		colors.put("PINK",Color.PINK);
		colors.put("ORANGE",Color.ORANGE);
		colors.put("MAGENTA",Color.MAGENTA);
		colors.put("LIGHT_GRAY",Color.LIGHT_GRAY);
		colors.put("GREEN",Color.GREEN);
		colors.put("DARK_GRAY",Color.DARK_GRAY);

		if( colors.containsKey(color) )
		{
			return colors.get(color);
		}
		else
		{
			return null;
		}
	}

	private static Color getColorValue(Color curVal, String newVal)
	{
		Color foundColor = getColorByName(newVal);

		if( foundColor != null )
		{
			return foundColor;
		}
		else
		{
			try
			{
				JSONArray temp2 = new JSONArray(newVal);
				Integer r = (Integer)temp2.get(0);
				Integer g = (Integer)temp2.get(1);
				Integer b = (Integer)temp2.get(2);
				Color newcolor = new Color(r.intValue(),g.intValue(),b.intValue());
				return newcolor;
			}
			catch(Exception e)
			{
				return curVal;
			}
		}
	}//getColor
	
	private static ArrayList<Color> getColorArray(ArrayList<Color> curVal, String newval)
	{
		try
		{
			JSONArray parts = new JSONArray(newval);
			ArrayList<Color> newlist = new ArrayList<Color>();
			for(int i=0;i<parts.length();i++)
			{
				Object temp = parts.get(i);

				if( temp instanceof String)
				{
					Color newcolor = getColorByName(parts.getString(i));
					newlist.add(newcolor);
				}
				else if( temp instanceof JSONArray )
				{
					JSONArray temp2 = parts.getJSONArray(i);
					Integer r = (Integer)temp2.get(0);
					Integer g = (Integer)temp2.get(1);
					Integer b = (Integer)temp2.get(2);
					Color newcolor = new Color(r.intValue(),g.intValue(),b.intValue());
					newlist.add(newcolor);
				}
				else
				{
					System.out.println("Error: \"" + temp + "\" is not a valid color.");
				}
			}
			return newlist;
		}
		catch(Exception e)
		{
			System.out.println("Error: You specified an invalid color list.");
			return curVal;
		}
	}

	private static ArrayList<String> getStringArray(ArrayList<String> curVal, String newval)
	{
		try
		{
			JSONArray parts = new JSONArray(newval);
			ArrayList<String> newlist = new ArrayList<String>();
			for(int i=0;i<parts.length();i++)
			{
				newlist.add( parts.getString(i) );
			}
			return newlist;
		}
		catch(Exception e)
		{
			System.out.println("Error: Invalid string list ("+newval+")");
			return curVal;
		}
	}

	private static ArrayList<Integer> getIntArray(ArrayList<Integer> curVal, String newval)
	{
		try
		{
			JSONArray parts = new JSONArray(newval);
			ArrayList<Integer> newlist = new ArrayList<Integer>();	
			for(int i=0;i<parts.length();i++)
			{
				newlist.add( (Integer) parts.get(i) );
			}
			return newlist;
		}
		catch(Exception e)
		{
			System.out.println("Error: Invalid integer list ("+newval+")");
			return curVal;
		}	
	}//getIntArray
	
	private static ArrayList<Double> getDoubleArray(ArrayList<Double> curVal, String newval)
	{
		
		try
		{
			JSONArray parts = new JSONArray(newval);
			ArrayList<Double> newlist = new ArrayList<Double>();	
			for(int i=0;i<parts.length();i++)
			{
				newlist.add( (Double) parts.get(i) );
			}
			return newlist;
		}
		catch(Exception e)
		{
			System.out.println("Error: Invalid double list ("+newval+")");
			return curVal;
		}	
	}//getDoubleArray



}//End Options
