package ics.uci.edu.Calico.UI;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;

public class DragHistory {
	
	private boolean enclosed = false;
	public ArrayList<Point2D> path;
	private int beginIndex = 0, endIndex = 0;
	
	
	
	public DragHistory() {
		path = new ArrayList<Point2D>();
	}
	
	public void add(Point2D p) {
		path.add(0, p);
	}
	
	/**
	 * Will check if the path is enclosed. 
	 * If the path ever becomes enclosed, the result is cached so it doesn't
	 * have to be calculated again.
	 * @return If the path of the DragHistory intersects itself.
	 */
	public boolean isEnclosed() {
		if (enclosed)
			return true;
		
		for (int i = 1; i < path.size() - 2; i++) {
			for (int j = i + 2; j < path.size() -1; j++) {
				if (intersect(path.get(i-1), path.get(i), path.get(j-1), path.get(j))) {
					if (Math.abs(j - i) < 10)
						continue;					
					enclosed = true;
					beginIndex = i-1;
					endIndex = j;
					break;
				}
			}
			if (enclosed)
				break;
		}
		
		return enclosed;
	}
	
	public int getNumPoints() {
		return path.size();
	}
	
	/**
	 * This makes my life easier by allowing to see if two lines composed of Point2D objects intersect.
	 * @param l1p1
	 * @param l1p2
	 * @param l2p1
	 * @param l2p2
	 * @return If they intersect.
	 */
	public static boolean intersect(Point2D l1p1, Point2D l1p2, Point2D l2p1, Point2D l2p2) {
		return Line2D.linesIntersect(l1p1.getX(), l1p1.getY(), l1p2.getX(), l1p2.getY(),
									 l2p1.getX(), l2p1.getY(), l2p2.getX(), l2p2.getY());
	}
	
	public static boolean intersect(Polygon p1, Polygon p2) {
		for (int i = 1; i < p1.npoints; i++) {
			for (int j = 1; j < p2.npoints; j++) {
				if (Line2D.linesIntersect(p1.xpoints[i-1], p1.ypoints[i-1], p1.xpoints[i], p1.ypoints[i], 
						p2.xpoints[j-1], p2.ypoints[j-1], p2.xpoints[j], p2.ypoints[j]))
					return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns the path of DragHistory as a polygon regardless of path history.
	 * @return
	 */
	public Polygon getPolyline() {
		int pathsize = path.size();

		int[] xArr = new int[pathsize];
		int[] yArr = new int[pathsize];
		
		for (int i = 0; i < pathsize; i++) {
			xArr[i] = new Double(path.get(i).getX()).intValue();
			yArr[i] = new Double(path.get(i).getY()).intValue();
		}
		return new Polygon(xArr, yArr, pathsize );
	}	
	
	private int[] getXArray() {
		int[] xArr = new int[endIndex - beginIndex];
		int i = beginIndex;
		while (i < endIndex) {
			xArr[i++ - beginIndex] = new Double(path.get(i).getX()).intValue();
		}
		return xArr;
	}
	
	private int[] getYArray() {
		int[] yArr = new int[endIndex - beginIndex];
		int i = beginIndex;
		while (i < endIndex) {
			yArr[i++ - beginIndex] = new Double(path.get(i).getY()).intValue();
		}
		return yArr;
	}
	
	/**
	 * This should never be used on a polygon that is never completed. If the polygon
	 * isn't completed, it will return null;
	 * @return
	 */
	public Polygon getPolygon() {
		if (!enclosed)
			return null;
		
		return new Polygon(getXArray(), getYArray(), endIndex - beginIndex);
	}
	
	
	

}
