package ics.uci.edu.Calico.UI;

import ics.uci.edu.Calico.Structures.*;
import ics.uci.edu.Calico.UI.ScrapPainter.ScrapWidgetItem;
import ics.uci.edu.Calico.UI.components.*;
import ics.uci.edu.Calico.UI.components.palettes.*;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.ScrapModel;
import ics.uci.edu.Calico.models.SetScrapTransparency;
import ics.uci.edu.Calico.models.actions.*;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.util.concurrent.*;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

//import com.sun.corba.se.pept.transport.Selector;

import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.util.PDimension;
import edu.umd.cs.piccolox.nodes.PLine;

/**
 * @author nmangano
 *
 */
//note, this class is a bit unwieldy.
public class MouseInputHandler extends PBasicInputEventHandler {


	public static int NULL = -1;
	public static int DRAW = 1;
	public static int SCRAP_DRAG = 2;
	public static int SCRAP = 4;
	public static int ZOOM = 5;
	public static int FIGURE_BAR = 6;
	public static int SCRAP_OUTLINE = 7;
	public static int SCRAP_WIDGET = 8;
	public static int ERASE = 9;
	public static int UNDO = 10;
	public static int GENERIC_WIDGET = 11;
	public static int SCRAP_ERASE = 12;

	private boolean mousePressed = false;

	public HashMap<Integer,Point2D> dragCoords;

	protected DragHistory hist;
	protected BGElementPainter activeLine;
	protected StrokeEvent activeStrokeEvent;

	//This is mainly to see if it was just a click.
	private boolean cameraElem = false;
	private int semaphore = 0;

	private CalicoCanvas canvas;

	private PNode cache;
	private ScrapPainter eraseCache;

	private Point2D mDown;
	private Point2D mDownCanvas;
	private double zoomScale = 1;

	//keep this cached so we can close a scrap widget on next click
	private ScrapPainter openedWidget;

	private ExecutorService threadManager;

	private int MODE = NULL;

	public MouseInputHandler(CalicoCanvas c) {
		super();
		canvas = c;
		threadManager = Executors.newFixedThreadPool( 1 );
		dragCoords = new HashMap<Integer,Point2D>();
	}

	//The mode is set on mouse down.
	public void mousePressed(PInputEvent e) {
		PLayer layer = ((PCanvas)(e.getComponent())).getLayer();
		
		
		if (mousePressed)
			return;
		mousePressed = true;

		if (semaphore < 0) {
			System.out.println("MouseInputHandler:mousePressed:Semaphore: " + semaphore);
			
		}
		if (semaphore > 0) {
			System.out.println("MouseInputHandler:mousePressed:Semaphore: " + semaphore);
			layer.removeChild(activeLine);
		}
		activeLine = new BGElementPainter();
		semaphore++;
	
		activeStrokeEvent = new StrokeEvent(e);
		
		mDown = e.getPosition();
		mDownCanvas = e.getCanvasPosition();

		if (openedWidget != null) {

			ScrapWidgetItem item = openedWidget.getWidgetItem(e.getPosition());
			if (item != null) {
				if (item.getText().compareTo(ScrapWidgetItem.DROP) == 0)
					canvas.applyAction(new DropScrapToBackground(openedWidget.getModel().id));
				else if (item.getText().compareTo(ScrapWidgetItem.DUPLICATE) == 0) {
					canvas.applyAction(new DuplicateScrap(openedWidget.getModel().id));
				} else if (item.getText().compareTo(ScrapWidgetItem.RECTANGLE) == 0) {
					canvas.applyAction(new TransformScrapToRect(openedWidget.getModel().id));
				} else if (item.getText().compareTo(ScrapWidgetItem.TRANSPARENT) == 0) {
					canvas.applyAction(new SetScrapTransparency(openedWidget.getModel().id, true));
				} else if (item.getText().compareTo(ScrapWidgetItem.OPAQUE) == 0) {
					canvas.applyAction(new SetScrapTransparency(openedWidget.getModel().id, false));
				}
				MODE = SCRAP_WIDGET;
			}

			openedWidget.closeWidgetMenu();
			openedWidget = null;
		}

		if (isCameraLayerElem(e.getCanvasPosition())) {
			cameraElem = true;
			return;
		}
		if (isGenericWidget(e.getCanvasPosition())) {
			MODE = GENERIC_WIDGET;
			return;
		}
		else if (e.isMiddleMouseButton()) {
			eraseCache = getScrapAt(e.getPosition());
			if (eraseCache!= null) {
				MODE = SCRAP_ERASE;
			}
			else {
				eraseCache = null;
				MODE = ERASE;
			}
			
		}
		else if (MODE == SCRAP_WIDGET) {
			//do nothing
		}
		//MODE: ZOOM
		else if (getZoom(e) != null) {
			MODE = ZOOM;
			cache = getZoom(e);
		}
		//MODE: FIGURE_BAR
		else if (getFigBarAt(e) != null) {
			MODE = FIGURE_BAR;
			cache = getFigBarAt(e);
		}
		//MODE: SCRAP
		else if (getScrapAt(e.getPosition()) != null) {
			MODE = SCRAP;
			cache = getScrapAt(e.getPosition());
			ScrapPainter painter = (ScrapPainter)cache;


			if (e.isRightMouseButton()) {

				activeStrokeEvent.occludingScraps = canvas.getOccluding(painter);
				activeStrokeEvent.occludingScraps.add(0, painter);
				for (ScrapPainter sp : activeStrokeEvent.occludingScraps) {
					sp.setTranslucent(true);
				}
				int[] scrapsToRaise = new int[activeStrokeEvent.occludingScraps.size()];
				for (int i = 0; i < activeStrokeEvent.occludingScraps.size(); i++) {
					scrapsToRaise[i] = activeStrokeEvent.occludingScraps.get(i).getModel().id;
					dragCoords.put(Integer.valueOf(activeStrokeEvent.occludingScraps.get(i).getModel().id), 
							ScrapController.getMidPoint(activeStrokeEvent.occludingScraps.get(i).getModel()));
				}
				canvas.applyAction(new ScrapsRaised(scrapsToRaise));
				canvas.moveToFront(painter, null);
				MODE = SCRAP_DRAG;
				
				
			}
			else if (painter.widgetContains(e.getPosition())) {
				MODE = SCRAP_WIDGET;
				if (openedWidget != null) {
					openedWidget.closeWidgetMenu();
				}
				openedWidget = painter;
				openedWidget.openWidgetMenu();
			}
		}
		//MODE: DRAG
		else if (e.isRightMouseButton()) {
			MODE = SCRAP_OUTLINE;
		}
		//MODE: DRAW
		else {
			MODE = DRAW;
		}

		if (e.isLeftMouseButton() && (MODE == DRAW || MODE == SCRAP)) {
			
			activeLine.setStroke(new BasicStroke(1.0f));
			activeLine.setColor(canvas.getPenColor());
			layer.addChild(activeLine);
		}
		else if (MODE == SCRAP_OUTLINE) {
			
			float dash1[] = {10.0f};
			BasicStroke dashed = new BasicStroke(3.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);
			activeLine.setStroke(dashed);
			layer.addChild(activeLine);
		}

		hist = new DragHistory();

		updatePoly(e.getPosition());
	}

	private boolean isGenericWidget(Point2D p) {
		for (Object node : canvas.getCamera().getAllNodes())
			if ((node instanceof Panning ||
					node instanceof CalicoGrid ||
					node instanceof ZoomSlider ||
					node instanceof Undo) &&
					((PNode)node).getBounds().contains(p))
					return true;
		return false;
	}

	public void mouseDragged(PInputEvent e) {
		if (cameraElem || MODE == MouseInputHandler.GENERIC_WIDGET)
			return;

		updatePoly(e.getPosition());
		activeStrokeEvent.dragged = true;

		//Code to handle zoom
		if (MODE == ZOOM) {


			Zoom zoom = (Zoom)cache;
			Rectangle2D b = zoom.getBounds();
			Point2D bCenter = new Point2D.Double(b.getCenterX(), b.getCenterY());

			Point2D p = e.getPositionRelativeTo(zoom);
			PDimension d = e.getDeltaRelativeTo(zoom);
			double deltaX = d.width;
			double deltaY = d.height;

			if (
				/*upper left quadrant*/
				p.getX() < bCenter.getX() && p.getY() < bCenter.getY() &&
					deltaX > 0 && deltaY < 0 ||
				/*upper right quadrant*/
				p.getX() > bCenter.getX() && p.getY() < bCenter.getY() &&
					deltaX > 0 && deltaY > 0 ||
				/*bottom right quadrant*/
				p.getX() > bCenter.getX() && p.getY() > bCenter.getY() &&
					deltaX < 0 && deltaY > 0 ||
				/*/bottom left quadrant*/
				p.getX() < bCenter.getX() && p.getY() > bCenter.getY() &&
					deltaX < 0 && deltaY < 0) {
				zoom.setPoint(e.getCanvasPosition());
				zoomScale *= 1.05;
				zoom.setZoomScale(zoomScale);
				e.getCamera().scaleViewAboutPoint(1.05, ((PCanvas)(e.getComponent())).getWidth()/2,
						((PCanvas)(e.getComponent())).getHeight()/2);
				zoom.repaint();
			} else if (
				/*upper left quadrant*/
				p.getX() < b.getCenterX() && p.getY() < b.getCenterY() &&
					deltaX < 0 && deltaY > 0 ||
				/*upper right quadrant*/
				p.getX() > b.getCenterX() && p.getY() < b.getCenterY() &&
					deltaX < 0 && deltaY < 0 ||
				/*bottom right quadrant*/
				p.getX() > b.getCenterX() && p.getY() > b.getCenterY() &&
					deltaX > 0 && deltaY < 0 ||
				/*/bottom left quadrant*/
				p.getX() < b.getCenterX() && p.getY() > b.getCenterY() &&
					deltaX > 0 && deltaY > 0) {
				zoom.setPoint(e.getCanvasPosition());
				zoomScale *= .95;
				zoom.setZoomScale(zoomScale);
				e.getCamera().scaleViewAboutPoint(.95, ((PCanvas)(e.getComponent())).getWidth()/2,
						((PCanvas)(e.getComponent())).getHeight()/2);
				zoom.repaint();
			}


		}
		//Code for erase
		else if (MODE == ERASE) {
			Rectangle2D eraserBounds = new Rectangle2D.Double(e.getPosition().getX() - 10, e.getPosition().getY() - 10, 20, 20);
			for (Object node : canvas.getLayer().getAllNodes()) {
				if (node instanceof BGElementPainter &&
						BGElementController.intersects(((BGElementPainter)node).getModel(), eraserBounds)) {
					canvas.applyAction(new BGEErase(((BGElementPainter)node).getId(), eraserBounds));
					break;
					//BGElementController.trim((BGElementPainter)node, eraserBounds);
				}
			}
		}
		//Code for scrap erase
		else if (MODE == SCRAP_ERASE) {
			Rectangle2D eraserBounds = new Rectangle2D.Double(e.getPosition().getX() - 10, e.getPosition().getY() - 10, 20, 20);
			for (int i = 0; i < eraseCache.getModel().encasedFigures.size(); i++) {
				if (ScrapController.intersectsFigure(eraseCache.getModel(), eraserBounds)) {
					canvas.applyAction(new SFErase(eraseCache.getModel().id, eraserBounds));
					break;
				}
			}
		}
		//Code to drag scraps
		else if (MODE == SCRAP_DRAG /* && mouseEventQueue.lastElement().ev.isRightMouseButton()*/) {
			ScrapPainter painter = (ScrapPainter)cache;
			if (!e.isRightMouseButton())
				return;


			int deltaX = new Double(e.getDelta().getWidth()).intValue();
			int deltaY = new Double(e.getDelta().getHeight()).intValue();

			painter.translate(deltaX, deltaY);
			for (ScrapPainter sp : activeStrokeEvent.occludingScraps) {
				if (sp.getDrawingPriority() > painter.getDrawingPriority()) {
					sp.translate(deltaX, deltaY);
					sp.setBounds();
				}
			}
			
			painter.setBounds();
		}
	}


	public void mouseReleased(PInputEvent e) {
		if (!mousePressed)
			return;
		mousePressed = false;

		
		semaphore--;
		
		if (semaphore > -1) {
			MouseUpTask task = new MouseUpTask(activeLine, activeStrokeEvent.ev, e, activeStrokeEvent.mDownPos, 
					activeStrokeEvent.dragged, hist, MODE, activeStrokeEvent.occludingScraps);
			threadManager.execute(task);
		}
		MODE = -1;

	}

	private class MouseUpTask implements Runnable {
		
		BGElementPainter line;
		PInputEvent e;
		PInputEvent upEv;
		Point2D mDPos;
		boolean dragged;
		DragHistory hist;
		int MODE;
		ArrayList<ScrapPainter> occludingScraps;
		
		public MouseUpTask(BGElementPainter l, PInputEvent pie, PInputEvent ue, Point2D mdp, boolean d, DragHistory h, int m, ArrayList<ScrapPainter> os) {
			line = l;
			e = pie;
			upEv = ue;
			mDPos = mdp;
			dragged = d;
			hist = h;
			MODE = m;
			occludingScraps = os;
		}

		public void run() {
			boolean wasDragToFigbar = false;

			if (cameraElem) {
				cameraElem = false;
				return;
			}

			if (MODE == ERASE) {
				canvas.applyAction(new EraseBGElement());
			}
			else if (MODE == SCRAP_WIDGET) {
				return;
			}
			else if (MODE == FIGURE_BAR || MODE==SCRAP_DRAG)
			{
				if( MODE==SCRAP_DRAG)
				{
					if( getFigBarAt(upEv) != null )
					{
					
						
						wasDragToFigbar = true;
						Palettes bar = (Palettes)getFigBarAt(upEv);
						
						/*if(bar.isInHiddenMode)
						{
							bar.showHide();
						}*/
						
						ArrayList<ScrapPainter> scraps = new ArrayList<ScrapPainter>();
						for (int i = 0; i < occludingScraps.size(); i++)
						{
							if (occludingScraps.get(i) instanceof ScrapPainter )
							{
								scraps.add( (ScrapPainter)occludingScraps.get(i));

								Integer key = Integer.valueOf(((ScrapPainter)occludingScraps.get(i)).getModel().id);

								
								if(dragCoords.containsKey( key ) )
								{
									((ScrapPainter)occludingScraps.get(i)).translateTo(dragCoords.get(key));
									((ScrapPainter)occludingScraps.get(i)).setTranslucent(false);
									((ScrapPainter)occludingScraps.get(i)).setBounds();
									dragCoords.remove( key );
								}
							}
						}
						if( scraps.size() > 0 )
						{
							bar.registerScrap(scraps, e.getCanvasPosition());
						}
						else
						{
							bar.showHide();
						}
					}

				}
				else
				{

					Palettes bar = (Palettes)cache;
					if (distance(mDownCanvas, e.getCanvasPosition()) < 5)
					{
						cache = bar;
						if(bar.isInHiddenMode)
						{
							bar.showHide(); //opens here
						}
						else if (!bar.isInHiddenMode && bar.getIndex(e.getCanvasPosition()) == Palettes.PULL_HANDLE)
						{
							bar.showHide();
						}
						if(e.isRightMouseButton())
						{
							int index = bar.getIndex(e.getCanvasPosition());
							if(index!=-1)
							{
								bar.deleteCell(index);
							}
							System.out.println("INDEX: " + index);
						}
						else
						{

							ArrayList<ScrapPainter> scraps = new ArrayList<ScrapPainter>();

							for (int i = 0; i < canvas.getLayer().getChildrenCount(); i++)
							{
								if (canvas.getLayer().getChild(i) instanceof ScrapPainter )
								{
									scraps.add( (ScrapPainter)canvas.getLayer().getChild(i));
								}
							}

							if( scraps.size() > 0 )
							{
								bar.registerScrap(scraps, e.getCanvasPosition());
							}
							/*
							else
							{
								bar.showHide(); //closes it on startup
							}*/
						}

					} 
					else if (bar.getIndex(mDownCanvas) > -1 && bar.getIndex(e.getCanvasPosition()) > -1) {
						bar.movePaletteItem(bar.getIndex(mDownCanvas), bar.getIndex(e.getCanvasPosition()));
					}
					else {
						if (bar.getScrapAt(mDownCanvas, e.getPosition()) != null) {
							CellContents contents = bar.getScrapAt(mDownCanvas, e.getPosition());
							canvas.applyAction(new AddPremadeScrap(contents.getScraps(), contents.getRelationships()));
						}
					}
				}
			}
			if (MODE == ZOOM) {
				if (distance(mDown, e.getPosition()) < 5) {
					//reset camera (zoom and pan)
					e.getCamera().scaleViewAboutPoint(1/zoomScale, ((PCanvas)(e.getComponent())).getWidth()/2,
							((PCanvas)(e.getComponent())).getHeight()/2);
					zoomScale = 1;
					Zoom zoom = (Zoom)cache;
					zoom.setZoomScale(zoomScale);
					zoom.setPoint(new Point(0,0));
				}
			}

			if (MODE == DRAW || (MODE == SCRAP && e.isLeftMouseButton())) {

				PNode parent = (PNode)canvas.getLayer();
				if (attemptRelationship(parent, hist))
				{
					System.out.println("MouseInput:MouseUp:Draw:5");
				} 
				
				else if (attemptInternalDraw(e, hist))
				{
					System.out.println("MouseInput:MouseUp:Draw:3");
				}
				else if (attemptDeleteScrap(hist, parent))
				{
					System.out.println("MouseInput:MouseUp:Draw:4");
				}
				else if (attemptExtend(e, hist))
				{
					System.out.println("MouseInput:MouseUp:Draw:6");
				}
				else if (attemptModifyRel(e, hist)) 
				{
					System.out.println("MouseInput:MouseUp:Draw:1");
				}
				else if (attemptBGElement(e, hist))
				{
					System.out.println("MouseInput:MouseUp:Draw:2");
				} 
				else  {

				}


			}

			if (!wasDragToFigbar && MODE == SCRAP_DRAG) {

				ScrapModel model;
				Point2D midPoint;
				int[] scraps = new int[occludingScraps.size()],
					x = new int[occludingScraps.size()],
					y = new int[occludingScraps.size()];

				for (int i = 0; i < occludingScraps.size(); i++) {
					model = occludingScraps.get(i).getModel();
					midPoint = ScrapController.getMidPoint(model);
					scraps[i] = model.id;
					x[i] = new Double(midPoint.getX()).intValue();
					y[i] = new Double(midPoint.getY()).intValue();
				}

				canvas.applyAction(new TranslateScrap(x, y, scraps));
			}

			if (MODE == SCRAP_OUTLINE) {
				if (hist.getNumPoints() < 10) { }
					
				else if (hist.isEnclosed())
					canvas.applyAction(new AddScrap(hist.getPolygon()));
				else
					canvas.applyAction(new AddScrap(hist.getPolyline()));
			}


			//pop the draw stroke from the thread queue
			if (line != null && canvasContains(line))
				canvas.getLayer().removeChild(line);
			return;
		}

	}
	
	private boolean canvasContains(PNode node) {
		
		ListIterator<PNode> childrenIterator = canvas.getLayer().getChildrenIterator();
		while (childrenIterator.hasNext())
			if (childrenIterator.next().equals(node))
				return true;
		
		return false;
	}

	private boolean attemptBGElement(PInputEvent e, DragHistory hist) {
		for (Point2D p : hist.path)
			if (this.getScrapAt(p) != null)
				return false;


		canvas.applyAction(new CreateBackgroundElement(hist.getPolyline(), Color.black));
		return true;
	}

	private boolean attemptModifyRel(PInputEvent e, DragHistory hist) {
		Line2D line;
		Point2D intersect1 = null, intersect2 = null;
		ArrayList<RelationshipPainter> painters;
		RelationshipPainter painter = null;
		int inc = 2;
		int painterId = -1;

		painters = getIntersectingRels(hist.path);
		
		if (painters .size() < 1)
			return false;
		
		else if (painters.size() > 1) {
			
			int[] tempArr = new int[painters.size()];
			for (int i = 0; i < painters.size(); i++)
				tempArr[i] = painters.get(i).getModel().id;
			canvas.applyAction(new DeleteRelationship(tempArr));
		}
		
		else {		
			//this variable decides if we're thickening (true) or thinning (false) the relationship
			boolean thicken = true;
	
			//iterate through each segment of the line
			for (int i = 1; i < hist.path.size(); i++) {
				line = new Line2D.Double(hist.path.get(i-1).getX(), hist.path.get(i-1).getY(),
						hist.path.get(i).getX(), hist.path.get(i).getY());
				painter = (RelationshipController.intersects(painters.get(0).getModel(), line))?painters.get(0):null;
	
				
				if (painter != null) {
					if (painterId == -1)
						painterId = painter.getModel().id;
	
					if (intersect1 == null)
						intersect1 = RelationshipController.getIntersectingPoint(painter.getModel(), line);
					else if (intersect2 == null) {
						intersect2 = RelationshipController.getIntersectingPoint(painter.getModel(), line);
						//[v , <] -OR- [^ , >]
						if (RelationshipController.isAbove(painter.getModel(), hist.path.get(0)) && intersect1.getX() < intersect2.getX() ||
								!RelationshipController.isAbove(painter.getModel(), hist.path.get(0)) && intersect1.getX() > intersect2.getX())
							thicken = true;
						//[^ , <] -OR- [v , >]
						else
							thicken = false;
					}
					else
						inc++; 
				}
	
				painter = null;
			}
			if (inc == 2)
				canvas.applyAction(new DeleteRelationship(new int[] {painters.get(0).getModel().id}));	
			else if (thicken)
				canvas.applyAction(new ThickenRelationship(inc, painterId));
			else
				canvas.applyAction(new ThinRelationship(inc, painterId));
		}
		
		return true;
	}

	private boolean attemptRelationship(PNode parent, DragHistory hist) {
		Point2D p1 = hist.path.get(0);
		Point2D p2 = hist.path.get(hist.path.size()-1);
		ScrapPainter scrap1 = getScrapAt(p1);
		ScrapPainter scrap2 = getScrapAt(p2);
		if (scrap1 != null &&
				scrap2 != null &&
				!scrap1.getModel().equals(scrap2.getModel())) {
			boolean ret = canvas.applyAction(new AddRelationship(scrap1.getModel().id, scrap2.getModel().id, p1, p2));
			canvas.moveToFront(scrap1, null);
			return ret;
		}
		return false;
	}

	private boolean attemptExtend(PInputEvent e, DragHistory hist) {
		ScrapPainter painter = getScrapAt(e.getPosition());
		if (painter == null)
			return false;
		ScrapModel scrap = getScrapAt(e.getPosition()).getModel();



		if (ScrapController.containsPoint(scrap, hist.path.get(0)) &&
				ScrapController.containsPoint(scrap, hist.path.get(hist.path.size()-1))) {
			for (Point2D p : hist.path) {
				if (!ScrapController.containsPoint(scrap, p)) {

					return canvas.applyAction(new ExtendScrap(scrap.id, point2DtoPoly(hist.path)));
				}
			}
		}
		return false;
	}

	private Polygon point2DtoPoly(ArrayList<Point2D> pa) {
		int xArr[] = new int[hist.path.size()];
		int yArr[] = new int[hist.path.size()];
		for (int i = 0; i < hist.path.size(); i++) {
			xArr[i] = new Double(hist.path.get(i).getX()).intValue();
			yArr[i] = new Double(hist.path.get(i).getY()).intValue();
		}
		return new Polygon(xArr, yArr, hist.path.size());
	}

	private boolean attemptDeleteScrap(DragHistory hist, PNode parent) {
		//remove all intersecting concepts


		Polygon stroke = hist.getPolyline();
		ArrayList<Integer> toBeDeleted = new ArrayList<Integer>();
		if (getScrapAt(hist.path.get(0)) == null &&
			getScrapAt(hist.path.get(hist.path.size()-1)) == null)
				//find intersections
				for (int i = 0; i < canvas.getLayer().getChildrenCount(); i++) {
					if (canvas.getLayer().getChild(i) instanceof ScrapPainter &&
							ScrapController.intersects(((ScrapPainter)canvas.getLayer().getChild(i)).getTotalPoly(), stroke))
						toBeDeleted.add(new Integer(((ScrapPainter)canvas.getLayer().getChild(i)).getModel().id));
				}

		if (toBeDeleted.size() == 0)
			return false;
		//If they user deletes a selected concept, we're going to delete all the concepts
		//that are selected as well.
		boolean containsSelectable = false;
		ArrayList<ScrapPainter> selected = ScrapPainter.getSelected(parent);
		for (ScrapPainter sp: selected)
			for (Integer tbd : toBeDeleted)
				if (sp.getModel().equals(tbd.intValue()))
					containsSelectable = true;
		if (containsSelectable)
			for (ScrapPainter sp: selected)
				if (!toBeDeleted.contains(sp))
					toBeDeleted.add(new Integer(sp.getModel().id));

		if (toBeDeleted.size() > 0) {
			int[] toDelete = new int[toBeDeleted.size()];
			for (int i = 0; i < toDelete.length; i++)
				toDelete[i] = toBeDeleted.get(i).intValue();
			return canvas.applyAction(new DeleteScraps(toDelete));
		}

		else
			return false;
	}

	private boolean attemptInternalDraw(PInputEvent e, DragHistory hist) {
		ScrapPainter scrap = getScrapAt(e.getPosition());
		if (scrap == null)
			return false;

		for (Point2D p : hist.path) {
			if (!ScrapController.containsPoint(scrap.getModel(), p))
				return false;
		}

		return canvas.applyAction(new AddInternalScrap(scrap.getModel().id, point2DtoPoly(hist.path), Color.BLACK));
	}

	private ArrayList<Integer> getEncasedScraps(PInputEvent e, Polygon p) {
		ArrayList<Integer> ret = new ArrayList<Integer>();
		Collection col = canvas.getLayer().getAllNodes();
		for (Object o : col) {
			if (o instanceof ScrapPainter &&
					((ScrapPainter)o).intersects(p))
				ret.add(new Integer(((ScrapPainter)o).getModel().id));
		}

		return ret;
	}


//noChildLeftBehind(Har joke)
//>_>
	private boolean noChildAt(PInputEvent e) {
		for (int i = 0; i < canvas.getLayer().getChildrenCount(); i++) {
			for (Object node : canvas.getLayer().getAllNodes()) {
				if (node instanceof ScrapPainter &&
						ScrapController.containsPoint(((ScrapPainter)node).getModel(), e.getPosition()))
					return false;
				else if (node instanceof Bucket && ((Bucket)node).contains(e.getPosition()))
					return false;
			}
		}
		return true;
	}

	public void updatePoly(Point2D p) {
		activeLine.addPoint(new Double(p.getX()).intValue(), new Double(p.getY()).intValue());
		activeLine.repaint();
		hist.add(p);
	}

	protected void pan(PInputEvent e) {
		PCamera c = e.getCamera();
		Point2D l = e.getPosition();

		if (c.getViewBounds().contains(l)) {
			PDimension d = e.getDelta();
			c.translateView(d.getWidth(), d.getHeight());
		}
	}

	private ScrapPainter getScrapAt(Point2D p) {
		PLayer layer = canvas.getLayer();
		for (int i = layer.getChildrenCount()-1; i >= 0; i--) {
			if (layer.getChild(i) instanceof ScrapPainter &&
					((ScrapPainter)layer.getChild(i)).contains(p))
				return (ScrapPainter)layer.getChild(i);
		}
		return null;
	}

	private boolean isCameraLayerElem(Point2D p) {
		for (Object node : canvas.getCamera().getAllNodes())
			if ((node instanceof GridNavPointer ||
					node instanceof GridNavThumb) &&
					((PNode)node).getBounds().contains(p))
					return true;
		return false;
	}

	private Zoom getZoom(PInputEvent e) {
		for (Object node : canvas.getCamera().getAllNodes()) {
			if (node instanceof Zoom && ((Zoom)node).contains(e.getCanvasPosition()))
				return (Zoom)node;
		}
		return null;
	}

	private Palettes getFigBarAt(PInputEvent e) {
		for (Object node : canvas.getCamera().getAllNodes()) {
			if (node instanceof Palettes && ((Palettes)node).contains(e.getCanvasPosition()))
				return (Palettes)node;
		}
		return null;
	}

	private double distance(Point2D p1, Point2D p2) {
		return Math.sqrt(Math.pow(p2.getX()-p1.getX(), 2) + Math.pow(p2.getY()-p1.getY(), 2));
	}

	private ArrayList<RelationshipPainter> getIntersectingRels(ArrayList<Point2D> line) {
		ArrayList<RelationshipPainter> painters = new ArrayList<RelationshipPainter>();
		Line2D segment;
		for (int i = 1; i < line.size(); i++) {
			segment = new Line2D.Double(line.get(i), line.get(i-1));
			for (Object node : canvas.getLayer().getAllNodes()) {
				if (node instanceof RelationshipPainter &&
						RelationshipController.intersects(((RelationshipPainter)node).getModel(), segment) &&
						!painters.contains(node))
					painters.add((RelationshipPainter)node);
			}
		}
		
		return painters;
	}
	
	/**
	 * Returns the first RelationshipPainter who intersects with line.
	 * @param line
	 * @return
	 */
	private RelationshipPainter getIntersectingRel(Line2D line) {
		for (Object node : canvas.getLayer().getAllNodes()) {
			if (node instanceof RelationshipPainter &&
					RelationshipController.intersects(((RelationshipPainter)node).getModel(), line))
				return (RelationshipPainter)node;
		}

		return null;
	}

	private Palettes getPalettes() {
		for (Object node : canvas.getCamera().getAllNodes()) {
			if (node instanceof Palettes)
				return (Palettes)node;
		}
		return null;
	}
	
	private class StrokeEvent {
		public PInputEvent ev;
		public Point2D mDownPos;
		public boolean dragged = false;
		private ArrayList<ScrapPainter> occludingScraps;

		public StrokeEvent(PInputEvent e) {
			ev = e;
			mDownPos = ev.getPosition();
			occludingScraps = new ArrayList<ScrapPainter>();
		}
	}

}
