package ics.uci.edu.Calico.UI.components;

import ics.uci.edu.Calico.UI.CalicoCanvas;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Arc2D;

import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.util.PPaintContext;

public class GridNavThumb extends CalicoComponent {
	
	public static final int NORTH = 0;
	public static final int SOUTH = 1;
	public static final int EAST = 2;
	public static final int WEST = 3;
	
	public static int MOVE = 4;
	public static int COPY = 5;
	
	int direction;
	CalicoCanvas canvas;
	
	public GridNavThumb(int dir) {
		direction = dir;
		
		this.addInputEventListener(new NavEventListener());
	}
	
	public void paint(PPaintContext context) {
		Graphics2D g2 = context.getGraphics();
		g2.setStroke(new BasicStroke(2.0f));
		Color thumbColor = new Color(167, 167, 138);
		
		int x = new Double(getBounds().getX()).intValue();
		int y = new Double(getBounds().getY()).intValue();
		int width = new Double(getBounds().getWidth()).intValue();
		int height = new Double(getBounds().getHeight()).intValue();
		
		if (direction == NORTH) {
			g2.setPaint(Color.white);
			g2.fillArc(x, y - (height + height/2), width, height*2, 180, 180);
			g2.setPaint(thumbColor);
			g2.fillArc(x, y - (height + height/2), width, height*2, 180, 90);
			g2.drawArc(x, y - (height + height/2), width, height*2, 180, 90);
			g2.drawArc(x, y - (height + height/2), width, height*2, 270, 90);
		}
		else if (direction == SOUTH) {
			g2.setPaint(Color.white);
			g2.fillArc(x, y + (height/2), width, height*2, 0, 180);
			g2.setPaint(thumbColor);
			//g2.fillArc(x, y + (height/2), width, height*2, 0, 90);
			g2.drawArc(x, y + (height/2), width, height*2, 0, 90);
			g2.fillArc(x, y + (height/2), width, height*2, 90, 90);
			g2.drawArc(x, y + (height/2), width, height*2, 90, 90);
		}
		else if (direction == WEST) {
			g2.setPaint(Color.white);
			g2.fillArc(x - (width + width/2), y, width*2, height, 270, 180);
			g2.setPaint(thumbColor);
			g2.fillArc(x - (width + width/2), y, width*2, height, 270, 90);
			g2.drawArc(x - (width + width/2), y, width*2, height, 270, 90);
			g2.drawArc(x - (width + width/2), y, width*2, height, 0, 90);
		}
		else if (direction == EAST) {
			g2.setPaint(Color.white);
			g2.fillArc(x + (width/2), y, width*2, height, 90, 180);
			g2.setPaint(thumbColor);
			//g2.fillArc(x + (width/2), y, width*2, height, 90, 90);
			g2.drawArc(x + (width/2), y, width*2, height, 90, 90);
			g2.fillArc(x + (width/2), y, width*2, height, 180, 90);
			g2.drawArc(x + (width/2), y, width*2, height, 180, 90);
		}
	}
	
	private class NavEventListener extends PBasicInputEventHandler {
		
		public void mousePressed(PInputEvent e) {
			Arc2D.Double moveArc = new Arc2D.Double();
			Arc2D.Double copyArc = new Arc2D.Double();
			
			int x = new Double(getBounds().getX()).intValue();
			int y = new Double(getBounds().getY()).intValue();
			int width = new Double(getBounds().getWidth()).intValue();
			int height = new Double(getBounds().getHeight()).intValue();
			
			if (direction == NORTH) {
				moveArc.setArc(x, y - (height + height/2), width, height*2, 270, 90, Arc2D.PIE);
				copyArc.setArc(x, y - (height + height/2), width, height*2, 180, 90, Arc2D.PIE);
			} 
			else if (direction == SOUTH) {
				moveArc.setArc(x, y + (height/2), width, height*2, 0, 90, Arc2D.PIE);
				copyArc.setArc(x, y + (height/2), width, height*2, 90, 90, Arc2D.PIE);
			}
			else if (direction == EAST) {
				moveArc.setArc(x + (width/2), y, width*2, height, 90, 90, Arc2D.PIE);
				copyArc.setArc(x + (width/2), y, width*2, height, 180, 90, Arc2D.PIE);
				
			}
			else if (direction == WEST) {
				moveArc.setArc(x - (width + width/2), y, width*2, height, 0, 90, Arc2D.PIE);
				copyArc.setArc(x - (width + width/2), y, width*2, height, 270, 90, Arc2D.PIE);
			}

			CalicoGrid grid = null;
			for (CalicoComponent vbc : canvas.getCalicoComponents())
				if (vbc instanceof CalicoGrid)
					grid = (CalicoGrid)vbc;		
			if (grid == null)
				System.err.println("Cannot find VBGrid in VBCanvas, something is mega wrong!");
			
			if (moveArc.contains(e.getCanvasPosition()))
				grid.shiftCanvas(direction, MOVE);
			if (copyArc.contains(e.getCanvasPosition()))
				grid.shiftCanvas(direction, COPY);
		}
		
	}

	@Override
	public void registerCanvas(CalicoCanvas c) {
		canvas = c;
		
		boolean hasGrid = false;
		for (CalicoComponent vbc : canvas.getCalicoComponents())
			if (vbc instanceof CalicoGrid)
				hasGrid = true;
		
		if (!hasGrid) {
			System.err.println("VBCanvas must have VBGrid before this can be added!");
			canvas.getCamera().removeChild(this);
		}		
	}

	@Override
	public boolean updateBounds(int width, int height) {
		switch (direction) {
		case NORTH: setBounds(width/2 - 40, 0, 80, 20);
			return true;
		case SOUTH: setBounds(width/2 - 40, height - 20, 80, 20);
			return true;
		case WEST: setBounds(0, height/2 - 40, 20, 80);
			return true;
		case EAST: setBounds(width - 20, height/2 - 40, 20, 80);
			return true;
		}
		return false;
	}
	
	
	
}
