package ics.uci.edu.Calico.UI.components;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.util.PPaintContext;

/**
 * Navigator will be a small circle that will show the real time reflection of everything
 * that takes place in the workspace. It will update itself every time the mouse is released
 * in the main workspace.
 * 
 * Navigator will also navigate between several levels of abstraction. The outer most level of abstraction
 * will be a the top. When navigating between levels of abstraction, the bottom most level of abstarction will
 * always be visible, unless another tree in the abstraction is visible.
 * @author nmangano
 *
 */
public class Navigator extends CalicoComponent
						implements CalicoActionListener {
	
	Image preview;
	
	public Navigator() {
		addInputEventListener(new NavigatorEventListener());
	}
	
	public boolean setBounds(double x, double y, double w, double h) {

		if (super.setBounds(x,y,w,h))
			return true;
		
		return false;
	}
	
	
	
	public void paint(PPaintContext aPaintContext) {
		Graphics2D g2 = aPaintContext.getGraphics(); 
		g2.setPaint(getPaint());
		
		Rectangle2D bounds = getBounds();
		
		g2.setColor(new Color(115,163,181));			
		g2.fillOval(new Double(bounds.getX()).intValue(), new Double(bounds.getY()).intValue(), 
				new Double(bounds.getWidth()).intValue(), new Double(bounds.getHeight()).intValue());
		g2.setColor(Color.WHITE);
		
		if (preview != null)	
			g2.drawImage(preview, new Double(bounds.getX()).intValue() + 15, new Double(bounds.getY()).intValue() + 15, null);
		
		GradientPaint gp = new GradientPaint(75, 75, Color.white,
		        95, 95, Color.gray, true);		
		g2.setPaint(gp);
		g2.setStroke(new BasicStroke(4.0f));	
		g2.drawOval(new Double(bounds.getX()).intValue(), new Double(bounds.getY()).intValue(), 
				new Double(bounds.getWidth()).intValue(), new Double(bounds.getHeight()).intValue());
		g2.setStroke(new BasicStroke(1.0f));
	}

	public void clearImage() {
		preview = null;
	}

	public void fireCalicoListener(CalicoActionEvent vbe) {
		preview = vbe.getEventSource().getImage();
		//VBGridMenu.saveImage((BufferedImage)preview, "previewUnresized.jpg");
		preview = CalicoCanvas.getProportionateScaledImage(preview, new Double(getBounds().getWidth()).intValue() - 30, 
				new Double(getBounds().getHeight()).intValue() - 30, Image.SCALE_SMOOTH);
		preview = preview.getScaledInstance(new Double(getBounds().getWidth()).intValue() - 30, new Double(getBounds().getHeight()).intValue() - 30, Image.SCALE_SMOOTH);
		repaint();
	}
	
	private class NavigatorEventListener extends PBasicInputEventHandler {
		
		public NavigatorEventListener() {
			
		}
		
		public void mousePressed(PInputEvent e) {
			System.out.println("Woowuu");
		}	
	}

	@Override
	public boolean updateBounds(int width, int height) {
		if (setBounds(width - 102, 2, 100, 100))
			return true;
		return false;
	}

	@Override
	public void registerCanvas(CalicoCanvas canvas) {
		canvas.addCalicoActionListener(this);
	}
	
}
