package ics.uci.edu.Calico.UI.components;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;

import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.util.PPaintContext;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.components.CalicoGrid.CGCell;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.PenColorChanged;

public class PenColorChooser extends CalicoComponent {

	Color[] colors;
	int activeColor;
	CalicoCanvas canvas;
	int circleOffset = 10;
	int circleSize = 20;
	int xOffset = 20;
	int yOffset = 5;
	
	Ellipse2D cir1, cir2, cir3, cir4;

	
	public PenColorChooser() {
		this(Color.BLACK);
	}
	
	public PenColorChooser(Color def) {
		this(Color.black, Color.black, Color.blue, Color.red, new Color(0, 150, 50));
	}
	
	public PenColorChooser(Color def, Color c0, Color c1, Color c2, Color c3) {		
		colors = new Color[4];
		colors[0] = c0;
		colors[1] = c1;
		colors[2] = c2;
		colors[3] = c3;
		
		if (def.equals(c0))
			activeColor = 0;
		else if (def.equals(c1))
			activeColor = 1;
		else if (def.equals(c2))
			activeColor = 2;
		else if (def.equals(c3))
			activeColor = 3;
		else {
			System.err.println("Default color does not match any existing colors, defaulting to first Color");
			activeColor = 0;
		}
		
		this.addInputEventListener(new PCCListener());
		
	}
	
	public void setColor(int c) {
		if (c < 0 || c > 3) {
			System.err.println("PenColorChooser:setColor - color choice " + c + " out of index");
			return;
		}
		activeColor = c;
		canvas.applyAction(new PenColorChanged(colors[c]));
		//System.out.println("New color selected:" + c);
	}
	
	
	
	public void paint(PPaintContext c) {
		Graphics2D g = c.getGraphics();
		
		Color col = g.getColor();

		
		g.setColor(colors[0]);
		g.fill(cir1);
		g.setColor(colors[1]);
		g.fill(cir2);
		g.setColor(colors[2]);
		g.fill(cir3);
		g.setColor(colors[3]);
		g.fill(cir4);

	}
	
	@Override
	public void registerCanvas(CalicoCanvas c) {
		canvas = c;
	}

	@Override
	public boolean updateBounds(int width, int height) {
		if (setBounds(xOffset, height - circleSize - yOffset, (circleSize+circleOffset)*4, circleSize))  {
			cir1 = new Ellipse2D.Double(xOffset + (circleSize+circleOffset)*0, height - circleSize - yOffset, circleSize, circleSize);
			cir2 = new Ellipse2D.Double(xOffset + (circleSize+circleOffset)*1, height - circleSize - yOffset, circleSize, circleSize);
			cir3 = new Ellipse2D.Double(xOffset + (circleSize+circleOffset)*2, height - circleSize - yOffset, circleSize, circleSize);
			cir4 = new Ellipse2D.Double(xOffset + (circleSize+circleOffset)*3, height - circleSize - yOffset, circleSize, circleSize);
			
			return true;
		}
		return false;
	}
	
	private class PCCListener extends PBasicInputEventHandler {
		
		public void mousePressed(PInputEvent e) {
			if (cir1.contains(e.getCanvasPosition()))
				setColor(0);
			else if (cir2.contains(e.getCanvasPosition()))
				setColor(1);
			else if (cir3.contains(e.getCanvasPosition()))
				setColor(2);
			else if (cir4.contains(e.getCanvasPosition()))
				setColor(3);
		}
	}

}
