package ics.uci.edu.Calico.UI.components;

import ics.uci.edu.Calico.Structures.RelationshipController;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.UndoAction;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import java.awt.Image;

import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.util.PPaintContext;

public class Undo extends CalicoComponent 
					implements CalicoActionListener {

	public int maxUndo = 50;
	private Area triangle;
	private Shape leftArrow, rightArrow;
	private Image undoImage, redoImage;
	private Rectangle undoImageBounds, redoImageBounds;
	private int xOffset = 10, yOffset = 10, spacing = 10;
	private boolean USE_PNG = true;
	
	
	private int undoIndex = 0;
	
	private ArrayList<ObjectHandlerModel> modelHistory;
	private CalicoCanvas canvas;
	
	public Undo() {
		
		modelHistory = new ArrayList<ObjectHandlerModel>();
		modelHistory.add(new ObjectHandlerModel());
		addInputEventListener(new UndoListener());
		ImageIcon uI = new ImageIcon("undo.png");
		undoImage = uI.getImage();
		ImageIcon rI = new ImageIcon("redo.png");
		redoImage = rI.getImage();
	}
	
	public boolean updateBounds(int canvasWidth, int canvasHeight) {
		if (USE_PNG) {
			undoImageBounds = new Rectangle(xOffset, yOffset, undoImage.getWidth(null), undoImage.getHeight(null));
			redoImageBounds = new Rectangle(xOffset + undoImageBounds.width + spacing, yOffset, redoImage.getWidth(null), redoImage.getHeight(null));
			
			if (setBounds(xOffset, yOffset, undoImageBounds.width + spacing + redoImageBounds.height, undoImageBounds.height)) {
				return true;
			}
			return false;
		} else {
			if (setBounds(20, 0, 70, 50)) {
				return true;
			}
			return false;
		}
	}
	
	public boolean setBounds(double x, int y, int w, int h) {
		
		if (super.setBounds(x,y,w,h)) {
			setupPolygons();
			return true;		
		}
		return false;
	}
	
	public void paint(PPaintContext context) {
		Graphics2D g2 = context.getGraphics();
		g2.setPaint(new Color(167, 167, 138));
		
		if (!USE_PNG) {
            g2.setStroke(new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND)); 
            g2.draw(leftArrow); 
            g2.draw(rightArrow); 
            if (canUndo()) 
            	g2.fill(leftArrow); 
            if (canRedo()) 
            	g2.fill(rightArrow); 
		}
		else {
			g2.drawImage(undoImage, undoImageBounds.x, redoImageBounds.y, null);
			g2.drawImage(redoImage, redoImageBounds.x, redoImageBounds.y, null);
		}
		
		
	}
	
	public ObjectHandlerModel getUndo() {
		if (canUndo())
			return modelHistory.get(undoIndex + 1);
		else
			return null;
	}
	
	public boolean canUndo() {
		return undoIndex < maxUndo && undoIndex + 3 < modelHistory.size();
	}
	
	public boolean canRedo() {
		return undoIndex > 0 ;
	}
	
	public ObjectHandlerModel getRedo() {
		
		if (canRedo())
			return modelHistory.get(undoIndex - 1);
		else
			return null;
	}

	@Override
	public void registerCanvas(CalicoCanvas c) {
		canvas = c;
		canvas.addCalicoActionListener(this);
	}

	/**
	 * This method is a little dangerous because it assumes who ever threw the undo event
	 * performed the canUndo() check.
	 */
	public void fireCalicoListener(CalicoActionEvent vbe) {
		
		if (vbe.getAction() instanceof UndoAction) {
			if (((UndoAction)vbe.getAction()).type == UndoAction.UNDO)
				undoIndex++;
			else if (((UndoAction)vbe.getAction()).type == UndoAction.REDO)
				undoIndex--;
			else
				System.err.println("Undo:fireCalicoListener - bad UndoAction type");
		}
		else {
			//if the user clicks undo several times, then performs a new action, then
			//we remove the possibility to redo.
			if (undoIndex > 0) {
				for (int i = 0; i < undoIndex; i++)
					modelHistory.remove(0);
			}
			undoIndex = 0;
			modelHistory.add(0, vbe.getEventSource().getModel());
			
			if (modelHistory.size() > maxUndo)
				modelHistory.remove(modelHistory.size() - 1);
		}
	}
	
	public boolean contains(Point2D p) {
		return triangle.contains(p);
	}
	
	private void setupPolygons() {
		Rectangle2D bounds = getBounds().getBounds();
		Rectangle2D leftArrowB;
		leftArrow = RelationshipController.getArrow(new Double(bounds.getX()).intValue(),
				new Double(bounds.getY() + bounds.getHeight()/2).intValue(),
				new Double(bounds.getX() + bounds.getWidth()/2 - 2).intValue(),
				new Double(bounds.getY() + bounds.getHeight()/2).intValue(), 20, 5, .5);
		leftArrowB = leftArrow.getBounds2D();
		leftArrow = AffineTransform.getRotateInstance(Math.PI).createTransformedShape(leftArrow);
		//leftArrowB = leftArrow.getBounds2D();
		leftArrow = AffineTransform.getTranslateInstance(leftArrowB.getWidth() + leftArrowB.getX() * 2, leftArrowB.getHeight() + leftArrowB.getY() * 2).createTransformedShape(leftArrow);
		rightArrow = RelationshipController.getArrow(new Double(bounds.getX() + bounds.getWidth()/2 + 2).intValue(),
				new Double(bounds.getY() + bounds.getHeight()/2).intValue(),
				new Double(bounds.getX() + bounds.getWidth()).intValue(),
				new Double(bounds.getY() + bounds.getHeight()/2).intValue(), 20, 5, .5);
	}
	
	private class UndoListener extends PBasicInputEventHandler {
		
		public void mousePressed(PInputEvent e) {
			if ((!USE_PNG)?leftArrow.contains(e.getCanvasPosition()):undoImageBounds.contains(e.getCanvasPosition())) {
				if (canUndo()) {
					canvas.applyAction(new UndoAction(getUndo(), UndoAction.UNDO));
				}
				else {
					JOptionPane.showMessageDialog(null, "I can undo no more.");
				}
			}
			else if ((!USE_PNG)?rightArrow.contains(e.getCanvasPosition()):redoImageBounds.contains(e.getCanvasPosition())) {
				if (canRedo()) {
					canvas.applyAction(new UndoAction(getRedo(), UndoAction.REDO));
				}
				else {
					JOptionPane.showMessageDialog(null, "I can redo no more.");
				}
			}
		}
		
	}
	
}
