package ics.uci.edu.Calico.classroom;

import ics.uci.edu.Calico.UI.MouseInputHandler;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.components.Navigator;
import ics.uci.edu.Calico.UI.components.Undo;
import ics.uci.edu.Calico.UI.components.Zoom;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;
import ics.uci.edu.Calico.networking.SocketClient;
import ics.uci.edu.Calico.networking.SocketMultiThreadedServer;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class ClassroomClientUI extends JFrame implements CalicoActionListener {
	
		private static final long serialVersionUID = 1L;
		private ClassroomSocketClient client;
		private JFrame connectionFrame;
		private JTextField serverAddressBox;
		private JTextField nicknameBox;
		private JButton connectButton;
		protected CalicoCanvas canvas;
		private String serverAddress;
		private String nickname;
		private GridBagConstraints c = new GridBagConstraints();
		protected ClassroomMenu menu;
		
		public ClassroomClientUI(){
			Dimension fullScreen = Toolkit.getDefaultToolkit().getScreenSize();
			canvas = new CalicoCanvas();
			menu = new ClassroomMenu();
			setPreferredSize(fullScreen);//orginally 600,610
			setLayout(new GridBagLayout());
			drawMenu();
			drawFrame();
			serverAddress = "";
			nickname = "";
			client = new ClassroomSocketClient(canvas, this);
			drawConnectionFrame();
		}
				
		protected void drawMenu(){
			menu.addConnect(this);
			menu.addDisconnect(this);
			menu.addFileBreak();
			menu.addSaveOption(canvas, canvas);
			menu.addLoadGridOption(canvas);
			menu.addFileBreak();
			//menu.addExitOption();
		}
		
		public void drawFrame(){
			
	   		setBackground(Color.DARK_GRAY);
	   		setTitle("Calico 2.0 Alpha Client");
	 
	   		c.fill = GridBagConstraints.HORIZONTAL;
			c.weightx = 1.0;
			c.weighty = 0;
			c.gridy = 0;
			c.gridwidth = 4;
			c.anchor = GridBagConstraints.FIRST_LINE_START;
			add(menu,c);
			
			// TODO fix resize
			c.fill = GridBagConstraints.BOTH;
			c.weightx = 1.0;
			c.weighty = 1.0;
			c.gridy = 1;
			c.anchor = GridBagConstraints.FIRST_LINE_START;
			add(canvas,c);	

			drawCanvas();
			pack();
			repaint();
	}

		

		public void drawConnectionFrame() {
			GridBagConstraints d = new GridBagConstraints();
			connectionFrame = new JFrame();
			
			JPanel ipAddress = new JPanel();
			JPanel nicknamePanel = new JPanel();
			
			JLabel ipAddressLabel = new JLabel("IP Address: ");
			JLabel nicknameLabel = new JLabel("Nickname:  ");
			
			
			connectionFrame.setTitle("Connect");
			connectionFrame.setLocation(new Point(225,250));//TODO
			connectionFrame.setLayout(new GridBagLayout());
			ipAddress.setLayout(new GridBagLayout());
			nicknamePanel.setLayout(new GridBagLayout());
			
			serverAddressBox = new JTextField(1);
	   		connectButton = new JButton();
	   		nicknameBox = new JTextField(1);
	   		
	   		connectionFrame.setPreferredSize(new Dimension(250,150));
	   		
			connectButton.setText("Connect");
	        connectButton.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                connect(evt);
	            }
	        });
	        

	        //serverAddressBox.setText("128.195.216.113");
	        serverAddressBox.setText(serverAddress);
	        serverAddressBox.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                connect(evt);
	            }
	        });
	        
	        nicknameBox.setText(nickname);
	        nicknameBox.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	            	connect(evt);
	            }
	        });
	        
	        nicknameBox.repaint();
	        nicknamePanel.add(nicknameLabel);
	        d.ipadx = 100;
	        d.insets = new Insets(5,20,5,5);
	        nicknamePanel.add(nicknameBox,d);
	        d.gridy=0;
	        connectionFrame.add(nicknamePanel,d);
	     
	        ipAddress.add(ipAddressLabel);
	        
	        d.ipadx = 100;
	        d.insets = new Insets(5,20,5,5);
	        ipAddress.add(serverAddressBox,d);
	        d.gridy=1;
	        connectionFrame.add(ipAddress,d);
	        
	       
	        
	        
			d.gridy = 2;
			d.ipadx = 100;
			connectionFrame.add(connectButton,d);

			connectionFrame.setAlwaysOnTop(true);
			connectionFrame.setVisible(true);
			connectionFrame.repaint();
			connectionFrame.pack();
			
			
		}
		
		public void drawNewCanvas(ObjectHandlerModel newCanvas){
			client.setCanvas(newCanvas);
		}
		
		private void drawCanvas(){
		    canvas.addCalicoActionListener(this);
			canvas.removeInputEventListener(canvas.getZoomEventHandler());
			canvas.removeInputEventListener(canvas.getPanEventHandler());
			canvas.addInputEventListener(new MouseInputHandler(canvas));
			addComponents();
		}
		
		protected void addComponents(){
			canvas.addComponent(new Navigator());
			canvas.addComponent(new Undo());
		}
	
		
		public void fireCalicoListener(CalicoActionEvent action){
				client.fireAction(action);
		}
		
		public void connect(java.awt.event.ActionEvent evt) {
			   if(client.isConnected()){
				 message("Already Connected!");
			   }
			   else{
				   serverAddress = serverAddressBox.getText();
				   nickname = nicknameBox.getText();
				   client.connect(serverAddress,nickname);
				   this.setTitle(nicknameBox.getText() + "'s Calico Client");
			   }   
		   }
		
		public void removeConnectFrame(){
			connectionFrame.dispose();
		}
		
		public  CalicoCanvas getCanvas(){
			return canvas;
		}
		
		public ClassroomMenu getMenu(){
			return menu;
		}
		
		 private void message(String message){
			   JFrame errorFrame = new JFrame();
			   JOptionPane.showMessageDialog(errorFrame,
					    message,
					    "Client Message",
					    JOptionPane.INFORMATION_MESSAGE);
			   
		   }
		 
		  public void warningMessage(){
			  //TODO
			  final JFrame incCanvasFrame = new JFrame();
			  JLabel warning1 = new JLabel("Premade Canvas incoming, current work will be deleted");
			  JLabel warning2 = new JLabel("Do you wish to save now?");
			  JButton yesButton = new JButton("Yes");
			  JButton noButton = new JButton("No");
			  JPanel yesNoPanel = new JPanel();
			  
			  
			  incCanvasFrame.setLocation(new Point(225,250));
			  incCanvasFrame.setLayout(new GridBagLayout());
			
			  GridBagConstraints c = new GridBagConstraints();
			  
			  c.gridx = 1;
			  c.gridy = 1;
			  c.gridwidth = 2;
			  incCanvasFrame.add(warning1,c);
			  
			  c.gridx = 1;
			  c.gridy = 2;
			  c.gridwidth = 2;
			  incCanvasFrame.add(warning2,c);
			  yesNoPanel.add(yesButton);
			  yesNoPanel.add(noButton);
			  c.gridx = 1;
			  c.gridy = 3;
			  c.gridwidth = 2;
			  incCanvasFrame.add(yesNoPanel,c);
			  
			  yesButton.addActionListener(new java.awt.event.ActionListener() {
		          public void actionPerformed(java.awt.event.ActionEvent evt) {
		          	client.saveCurrentCanvas();
		          	incCanvasFrame.dispose();
		          }
		      });
			  
			  noButton.addActionListener(new java.awt.event.ActionListener() {
		          public void actionPerformed(java.awt.event.ActionEvent evt) {
		          	client.dontSaveCurrentCanvas();
		          	incCanvasFrame.dispose();
		          }
		      });
			  
			  WindowListener l = new WindowAdapter() {
		          public void windowClosing(WindowEvent e) {
		        	  client.dontSaveCurrentCanvas();   
		          }
			  };
			  incCanvasFrame.setAlwaysOnTop(true);
			  incCanvasFrame.addWindowListener(l);
			  incCanvasFrame.pack();
			  incCanvasFrame.setVisible(true);

		  }
		  
		  public void disconnect(){
			  client.finalize();
		  }
		 
		  public void addResizeListener(){
			  this.addComponentListener(new ComponentListener(){
					public void componentHidden(ComponentEvent arg0) {}
					public void componentMoved(ComponentEvent e) {}
					public void componentResized(ComponentEvent e) {
						System.out.println("shit");
					}
					public void componentShown(ComponentEvent e) {}
					
				});
		  }
		  
		  public static void main(String[] args){
		        ClassroomClientUI frame = new ClassroomClientUI();
		        WindowListener l = new WindowAdapter() {
		                public void windowClosing(WindowEvent e) {
		                        System.exit(0);
		                }
		        };
		        //frame.connect(null);
		        frame.addWindowListener(l);
		        frame.pack();
		        frame.setVisible(true);
		  }
		  
		  //TEST
		  public void startTimer(){
			  client.setTimer();
		  }
	
	
	
}
