package ics.uci.edu.Calico.classroom;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.CalicoAction;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;
import ics.uci.edu.Calico.networking.NetworkPacket;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.net.*;

public class ClassroomClientWorker extends JFrame implements CalicoActionListener, Runnable {
   
	private static final long serialVersionUID = 1L;
	private CalicoCanvas canvas;
	private ObjectOutputStream out;
	private ObjectInputStream in;
	private NetworkPacket networkPacket;
	private CalicoAction action;
	private Socket client;
	private String nickname;
	private boolean isConnected;
	private boolean isExpanded;
	private boolean alive;
	
public ClassroomClientWorker(Socket cl) {
	  client = cl;
	  out = null;
	  in = null;
	  networkPacket = null;
	  action = null;
	  canvas = new CalicoCanvas();
	  isConnected = true;
	  isExpanded = false;
	  alive = true;
	  drawCanvas();
  }
  
  public void run(){
    int packetID = -1;
	
    try{
     out = new ObjectOutputStream(client.getOutputStream());
     in = new ObjectInputStream(client.getInputStream());
    } catch (IOException e) {
      System.out.println("in or out failed");
      System.exit(-1);
    }
    while(!client.isClosed()){//while client is open
      try{ 
    	  networkPacket = (NetworkPacket)in.readObject();
    	  packetID = networkPacket.getPacketID();
    	  if(packetID == networkPacket.CALICO_ACTION){
    		  action = networkPacket.getCalicoAction();
    		  performDrawAction(action);
    	  }
    	  else if(packetID == networkPacket.PREMADE_CANVAS){
    		  canvas.loadModel(networkPacket.getPremadeCanvas());
    		  System.out.println("canvas received");
    		  canvas.repaint();
    	  }
    	  else if(packetID == networkPacket.NICKNAME){
    		  nickname = networkPacket.getNickname();
    	  }
       } catch (IOException e) {
    	   	finalize();
       } catch (ClassNotFoundException e) {
			System.out.println("Class could not be found");
			e.printStackTrace();
			}
    	} 
  }

  
  private void drawCanvas(){
	  
	   canvas.setPreferredSize(new Dimension(110, 110));
	   canvas.removeInputEventListener(canvas.getZoomEventHandler());
	   canvas.removeInputEventListener(canvas.getPanEventHandler());
	   getContentPane().add(canvas);
	   canvas.addCalicoActionListener(this);
	   canvas.getCamera().scaleViewAboutPoint(.100, 0, 0);//Zooms out.  (Scale, width, height)
	   pack();
  }

  
  public void fireCalicoListener(CalicoActionEvent action){
	   CalicoAction actionObject = action.getAction();
	   	if(isConnected){
		          try {
		        	  	System.out.println("firing VB SERVER");
		        	  	NetworkPacket packet = new NetworkPacket(actionObject);
		        		out.writeObject(packet);
				} catch (IOException e) {
					System.out.println("Object could not be written");
					e.printStackTrace();
				}
	   	}
  }
  
  public void performDrawAction(CalicoAction action){
	  
	  CalicoActionListener actionListener = null;
	  for (CalicoActionListener listener : canvas.getListeners())
		  if (listener instanceof ClassroomClientWorker)
			  actionListener = listener;
	  
	  canvas.removeCalicoActionListener(actionListener);
	  canvas.applyAction(action);
	  canvas.repaint();
	  canvas.addCalicoActionListener(actionListener);
  }
  
  public void sendCanvas(ObjectHandlerModel toBeSent){
	  networkPacket = new NetworkPacket(toBeSent);
	  System.out.println("sending Canvas");
	  try {
		out.writeObject(networkPacket);
	} catch (IOException e) {
		System.out.println("Object could not be written");
		e.printStackTrace();
	}
  }
  
  public boolean ping(){
	  boolean flag = true;
      try {
    	  out.writeObject(null);
	} catch (IOException e) {
		flag =  false;
		isConnected = false;
	}
	return flag;
  }
  
  public void expanded(){
	  	isExpanded = true;
	}
	
	public void unExpand(){
		isExpanded = false;
	}
  
  public boolean isExpanded(){
	  return isExpanded;
  }
  
  public CalicoCanvas getVBCanvas(){
	  return canvas;
  }
  
  public String getNickname(){
	  return nickname;
  }
  
  public void setCanvas(CalicoCanvas vbCanvas){
	  	canvas = vbCanvas;
		//canvas.changeModel(vbCanvas.getModel());
		canvas.removeInputEventListener(canvas.getPanEventHandler());
		canvas.getCamera().scaleViewAboutPoint(.2, 0, 0);//.1,0,0
		canvas.repaint();
		isExpanded = false;
	}
  
  
  public boolean isConnected(){
	  return isConnected;
  }
  
	public void kill(){
		alive = false;
		isConnected = false;
	}
	
	public boolean isAlive(){
		return alive;
	}
  
  protected void finalize(){
	     try{
	         in.close();
	         out.close();
	    } catch (IOException e) {
	        System.out.println("Could not close.");
	        System.exit(-1);
	    }
	  }
  
  
}