package ics.uci.edu.Calico.classroom;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.networking.ClientWorker;

import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.Vector;

import javax.swing.*;

public class ClassroomServerUI extends JFrame{
	
	private static final long serialVersionUID = 1L;
	private JPanel grid;
	private java.awt.event.MouseAdapter enlargeCanvasEvent;
	private Vector<ClassroomClientWorker> canvasList;
	private ClassroomMenu menu;
	private GridBagConstraints c = new GridBagConstraints();
	private int frameX;
	private int frameY;
	private Dimension defaultCanvasSize;
	final static boolean RIGHT_TO_LEFT = false;
	
	public ClassroomServerUI(){
		setDefaultFrameAndCanvasSizes();
		drawMenu();
		drawFrame();
	}
	
	private void setDefaultFrameAndCanvasSizes(){
		int canvasX = 130;
		int canvasY = 110;
		
		frameX = (4*canvasX)+100;//80 because there are 8 10pixel spaces between the canvases
		frameY = (4*canvasY)+200;//89+40 to include the menu
		defaultCanvasSize = new Dimension(canvasX,canvasY);
	}
	
	
	
	private void drawMenu(){
		menu = new ClassroomMenu();
		menu.addEditor(this);
		//menu.addExitOption();
		repaint();
		pack();
	}
	
	
	 
	private void drawFrame(){
		
		grid = new JPanel();
		setTitle("VBoard 2.0 Alpha Server");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setPreferredSize(new Dimension(frameX,frameY));
		setVisible(true);
		
		setLayout(new GridBagLayout());

		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 0;
		c.gridy = 0;
		c.gridwidth = 4;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		add(menu,c);
		
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		grid.setBackground(Color.DARK_GRAY);
		add(grid,c);
		
		repaint();
		pack();
	}
	
	
	public void addToGrid(){
		Dimension currentD = this.getSize();
		setPreferredSize(currentD);
		
		grid.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		if (RIGHT_TO_LEFT) {
           grid.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        }
		grid.removeAll();
		
		double weightX = 0;
		int gridX = 0;
		int gridY = 0;
		int counter = 0;
		
		for(final ClassroomClientWorker cWorker : canvasList){
			counter++;
			if(cWorker.isConnected()){
				JLabel nickname = new JLabel();
				nickname.setText(cWorker.getNickname());
				c.weightx = weightX;
				c.gridx = gridX;
				c.gridy = gridY;
			
				
				c.weightx = weightX;
				c.gridx = gridX;
				c.gridy = gridY;
				c.insets = new Insets(10,10,10,10);//creates the gap between frames
				c.anchor = GridBagConstraints.FIRST_LINE_START;
				
				if(!cWorker.isExpanded()){
					CalicoCanvas canvas = cWorker.getVBCanvas();
					enlargeCanvasEvent = new java.awt.event.MouseAdapter() {
			            public void mouseClicked(java.awt.event.MouseEvent evt) {
			            	if(!cWorker.isExpanded()){
				            	cWorker.expanded();
				            	new ExpandedFrame(cWorker, canvasList);
			            	}
			            }
					};
					c.insets = new Insets(7,11,0,1);//(T,L,X,X)
					c.anchor = GridBagConstraints.FIRST_LINE_START;
					grid.add(nickname,c);
					
					c.insets = new Insets(10,10,10,10);
					canvas.addMouseListener(enlargeCanvasEvent);
					canvas.setPreferredSize(defaultCanvasSize);
					grid.add(canvas,c);
					
					
					
					grid.repaint();
					pack();
				}
				else{
					JLabel enlarged = new JLabel("Enlarged");//TODO ADD PICUTRE
					JPanel background = new JPanel();
					background.setPreferredSize(defaultCanvasSize);
					background.setBackground(Color.WHITE);
					background.add(enlarged);
					grid.add(background,c);
					grid.repaint();
					pack();
				}
				
				gridX++;
				if(gridX==4){
					gridY++;
					gridX=0;
				}
			}
		}
				
		JPanel panel = new JPanel();
	
		c.gridwidth = 4;
		c.anchor = GridBagConstraints.LAST_LINE_START;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridx = 0;
		panel.setPreferredSize(new Dimension(0,0));
		grid.add(panel,c);
		pack();
		repaint();
	}
	
	public void setCanvasList(Vector<ClassroomClientWorker> canvasList){
		this.canvasList = canvasList;
	}
	
	public Vector<ClassroomClientWorker> getActiveList(){
		return canvasList;
	}
	
	 public static void main(String[] args){
		ClassroomServerUI classroomUI = new ClassroomServerUI();
		new ClassroomServer(classroomUI);
	   }
}



class CanvasImage extends JComponent{
	
	private static final long serialVersionUID = 1L;
	Image image;
	
	public CanvasImage(Image image){
		this.image = image;
	}
	
}
