package ics.uci.edu.Calico.models;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.SimpleTimeZone;
import java.util.TimeZone;
import java.io.File;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.UI.components.CalicoGrid;
import ics.uci.edu.Calico.models.actions.CalicoAction;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.models.actions.ChangedCell;
import ics.uci.edu.Calico.models.actions.UndoAction;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;

public class CalicoLogger implements Serializable, CalicoActionListener{

	private static final long serialVersionUID = 1;
	private final static boolean FILE_SPLITING = true;
	private final static int MIN_ACTIONS_FOR_NEW = 15;
	private final static int MIN_SECONDS_FOR_NEW = 1200;
	private final static int MIN_VIABLE_ACTIONS = 6;
	private FileOutputStream fos;
	private ObjectOutputStream out;
	private int actionCount;
	private String fileName;
	private static String folderName = "Logs";
	private String pathName;
	private Date lastTimestamp;
	private int currentGridCell = -1;
	
	public CalicoLogger(){
		createHistory();
	}
	
	public void createHistory(){
		  actionCount = 0;
		  lastTimestamp = new Date(System.currentTimeMillis());
		
		  //Date timestamp = new Date(System.currentTimeMillis());
		  String[] ids = TimeZone.getAvailableIDs(-8 * 60 * 60 * 1000);
		  SimpleTimeZone pdt = new SimpleTimeZone(-8 * 60 * 60 * 1000, ids[0]);

		  // set up rules for daylight savings time
		  pdt.setStartRule(Calendar.MARCH, 1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
		  pdt.setEndRule(Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
		  
		  Calendar calendar = new GregorianCalendar(pdt);
		  Date currentTime = new Date();
		  calendar.setTime(currentTime);

		  int realMonth = calendar.get(Calendar.MONTH) + 1;
		  fileName = new String(
				  Snapshot.getVersion() + " - "+
				  realMonth+"-"+
				  calendar.get(Calendar.DAY_OF_MONTH)+"-"+
				  getSmallYear(calendar.get(Calendar.YEAR))+" - "+
				  get12HourTime(calendar.get(Calendar.HOUR_OF_DAY), calendar.get(Calendar.MINUTE))+".chf"
		  );
		  
		  System.out.println("Creating history file: " + fileName);
		  updatePathName();
		  
		  fos = null;
		  out = null;
		  try
		  {
			  if(!new File(folderName).exists()){
				  new File(folderName).mkdir();
			  }
			  fos = new FileOutputStream(pathName);
			  out = new ObjectOutputStream(fos);
		  }
		  catch(IOException ex)
		  {ex.printStackTrace();}
	}
	
	public String getSmallYear(int year){
		Integer i = new Integer(year);
		return i.toString().substring(2);
	}
	
	private void updatePathName(){
		pathName = folderName+File.separatorChar+fileName;
	}
	
	private String get12HourTime(int hour, int minute){
		String AMPM;
		String m;
		Integer h = new Integer(hour);
		
		if(hour > 11){
			AMPM = "PM";}
		else{
			AMPM = "AM";}		
		
		if(hour == 0){
			h = new Integer(12);}
		if(hour > 12){
			h = new Integer(hour - 12);}
		
		if(minute < 10){
			m = "0" + new Integer(minute).toString();}
		else{
			m = new Integer(minute).toString();}
		
		return h+"-"+m+AMPM;
	}
	
	public void fireCalicoListener(CalicoActionEvent vbe) {
		CalicoAction action = vbe.getAction();
		
		//Keep currentGridCell updated
		if (currentGridCell == -1){			
			currentGridCell = CalicoGrid.getStartingCell();
		}		
		
		if(action instanceof ChangedCell){
			currentGridCell = ((ChangedCell)action).getDestination();
		}
		
		CalicoCanvas tempCanvas = new CalicoCanvas();
	    tempCanvas.loadModel(vbe.getEventSource().getModel());	    
		writeSnapshot(new Snapshot(action, tempCanvas, currentGridCell) );
	}

	public void writeSnapshot(Snapshot snapshot){
		System.out.println("TESTTEST: "+ snapshot.getType());
		
		if(FILE_SPLITING){
			if(checkForNewFile(snapshot)){
				startNewFile();}}
		
		try
		  {
			  out.writeObject(snapshot);
			  //out.close(); //******************???
			  out.reset();
			  System.out.println("Writing action: " + ++actionCount);
		  }
		  //http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4088600
		  //http://forum.java.sun.com/thread.jspa?threadID=676568&messageID=3948919
		  //Heap problems: http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6525563
		  catch(IOException ex)
		  {ex.printStackTrace();}
		  	
	 }

	public boolean checkForNewFile(Snapshot snapshot){
		//Check if we have enough actions to bother sealing it off
		if(actionCount < MIN_ACTIONS_FOR_NEW){
			lastTimestamp = snapshot.getTimestamp();
			return false;
		}
		
		//Assuming we have enough actions, check to see if its been long enough
		Long currentTime = new Long(snapshot.getTimestamp().getTime());
		Long lastTime = new Long(lastTimestamp.getTime());
		Long difference = currentTime.longValue() - lastTime.longValue();
		
		//*************************************
		//System.out.println(difference);
		
		if(difference < MIN_SECONDS_FOR_NEW * 1000){
			lastTimestamp = snapshot.getTimestamp();
			return false;
		}
		
		//We have enough actions and its been long enough, return true
		lastTimestamp = snapshot.getTimestamp();
		
		return true;
		
		
	}
	
	private void startNewFile(){
		System.out.println("Creating new file!");
		closeStream();
		createHistory();
	}
	
	public void finalize(){
		//Right now this is a trivial function
		//but I'm leaving room for other stuff that I might want to do when closing later
		closeStream();
	}
	
	//Check to see if there are enough actions to even keep this log file
	private boolean checkViability(){
		if(actionCount - 2 < MIN_VIABLE_ACTIONS){
			return false;
		}
		return true;
	}
	
	public void closeStream(){
		try
		{
			out.flush();
			out.close();
		}
		catch(IOException ex)
		{ex.printStackTrace();}
		
		if(!checkViability()){
			new File(pathName).delete();
			return;
		}
		
		File oldFile = new File(pathName);
		
		//Remove the suffix
		fileName = new String(fileName.substring(0, fileName.length()-4));
		updatePathName();
		
		//Add the number of actions and re-add the suffix
		if(!oldFile.renameTo(new File(pathName + " - " + actionCount + " Actions.chf"))){
			System.out.println("Could not rename");
		}		
	}
	
	
}
