
package ics.uci.edu.Calico.models;

import java.util.ArrayList;

import ics.uci.edu.Calico.Structures.BGElementController;
import ics.uci.edu.Calico.Structures.RelationshipController;
import ics.uci.edu.Calico.Structures.ScrapController;
import ics.uci.edu.Calico.UI.BGElementPainter;
import ics.uci.edu.Calico.UI.RelationshipPainter;
import ics.uci.edu.Calico.UI.ScrapPainter;
import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.actions.AddInternalScrap;
import ics.uci.edu.Calico.models.actions.*;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;

import java.util.Date;
import java.awt.Image;
import java.awt.Color;
import java.awt.image.BufferedImage;

import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import ics.uci.edu.Calico.models.ObjectHandlerModel;

public class Snapshot implements Serializable{
	
	private static final long serialVersionUID = 2;
	public final static int ADD_SCRAP = 1;
	public final static int ADD_RELATIONSHIP = 2;
	public final static int DRAW = 3;
	public final static int SELECT_SCRAP = 4;
	public final static int DELETE_SCRAP = 5;
	public final static int MODIFY_SCRAP = 6;
	public final static int MODIFY_RELATIONSHIP = 7;
	public final static int SAVE = 8;
	public final static int LOAD = 9;
	public final static int GO_DOWN = 10;
	public final static int GO_UP = 11;
	public final static int MOVE_SCRAP = 12;
	public final static int DESELECT = 13;
	public final static int LOAD_STATE = 14;
	public final static int DELETE_RELATIONSHIP = 15;
	public final static int ADD_PREMADE = 16;
	public final static int CREATE_BACKGROUND_ELEMENT = 17;
	public final static int LIFT_SCRAP = 18;
	public final static int DROP_SCRAP = 19;
	public final static int DUPLICATE_SCRAP = 20;
	public final static int ERASE_BACKGROUND_ELEMENT = 21;
	public final static int TRANSFORM_SCRAP_TO_RECT = 22;
	
	public final static int BG_ERASE = 23;
	public final static int CHANGED_MODEL = 24;
	public final static int DROP_SCRAP_TO_BACKGROUND = 25;
	public final static int EXIT_CANVAS = 26;
	public final static int EXTEND_SCRAP = 27;
	public final static int UNDO_ACTION = 28;
	
	public final static int BG_COLOR_CHANGE = 29;
	public final static int PEN_COLOR_CHANGE = 30;
	
	public final static int CHANGE_CELL = 51;
	public final static int VIEW_GRID = 52;
	
	protected CalicoAction action;
	//private PLayer layer;
	protected ObjectHandlerModel model;
	protected int type;
	protected int gridCell;
	protected static String version = "3.0";
	
	public Date timestamp; 
	
	//**Not safe yet - Leaves me with a null action, need to be careful of this
	public Snapshot(){
		type = 0;
	}
	
	public Snapshot(CalicoAction newAction, CalicoCanvas newCanvas, int newGridCell) {
		PLayer layer = ((PLayer)newCanvas.getLayer());
		
		action = newAction;
		gridCell = newGridCell;
		generateType();
		timestamp = new Date(System.currentTimeMillis());
		model = new ObjectHandlerModel();
	
		//- - - - - - - -
		//Populating the objectHandlerModel
		PNode node;
		for (int i = 0; i < layer.getChildrenCount(); i++) {
			node = layer.getChild(i);
			if (node instanceof ScrapPainter)
				model.scraps.add(ScrapController.copy(((ScrapPainter)node).getModel()));
			else if (node instanceof RelationshipPainter)
				model.relationships.add(RelationshipController.copy(((RelationshipPainter)node).getModel()));
			else if (node instanceof BGElementPainter){
				model.bgElements.add(BGElementController.copy(((BGElementPainter)node).getModel()));
			}
			//else
				//System.err.println("VBCanvas:getModel() - Invalid node found");
		}
		model.objectIdTicker = newCanvas.getObjectIdTicker();
		//model.hierarchyId = newCanvas.hierarchyId;
		//model.hierarchyLevel = newCanvas.hierarchyLevel;
		//model.parentId = newCanvas.parentId;
		//- - - - - - - -
	}
	
	public int getType(){
		return type;
	}
	
	public CalicoAction getAction(){
		return action;
	}
	
	public static String getVersion(){
		return version;
	}
	
	public PLayer getLayer(){
		PLayer layer2 = new PLayer();
		
		if(model == null)
			model = new ObjectHandlerModel();
		
		ScrapPainter scrapPainter;
		for (ScrapModel s : model.scraps) {
			scrapPainter = new ScrapPainter(s);			
			layer2.addChild(scrapPainter);
			scrapPainter.setBounds();
		}
		RelationshipPainter relPainter;
		for (RelationshipModel r : model.relationships) {
			relPainter = new RelationshipPainter(r);
			relPainter.setBounds();
			layer2.addChild(relPainter);
			relPainter.moveToFront();
		}
		BGElementPainter bgPainter;
		for (BGElementModel b : model.bgElements) {
			bgPainter = new BGElementPainter(b);
			layer2.addChild(bgPainter);
			bgPainter.moveToBack();
		}
		return layer2;
	}
	
	public Date getTimestamp(){
		return timestamp;
	}
	

	
	public void generateType(){
		if (action instanceof AddScrap){
			type = ADD_SCRAP;
			return;}
		if(action instanceof AddInternalScrap){
			type = DRAW;
			return;}
		if (action instanceof SelectScraps){
			type = SELECT_SCRAP;
			return;}
		if (action instanceof ExtendScrap){
			type = MODIFY_SCRAP;
			return;}
		if (action instanceof DeleteScraps){
			type = DELETE_SCRAP;
			return;}
		if (action instanceof AddRelationship){
			type = ADD_RELATIONSHIP;
			return;}
		if (action instanceof ThickenRelationship || action instanceof ThinRelationship){
			type = MODIFY_RELATIONSHIP;
			return;}
		if (action instanceof LoadState){
			type = LOAD;
			return;}
		if (action instanceof TranslateScrap){
			type = MOVE_SCRAP;
			return;}
		if (action instanceof ChangedCell){
			type = CHANGE_CELL;
			return;}
		if (action instanceof GridView){
			type = VIEW_GRID;
			return;}
		if (action instanceof DeselectScraps){
			type = DESELECT;
			return;}
		if (action instanceof LoadState){
			type = LOAD_STATE;
			return;}
		if (action instanceof DeleteRelationship){
			type = DELETE_RELATIONSHIP;
			return;}
		if (action instanceof AddPremadeScrap){
			type = ADD_PREMADE;
			return;}
		if (action instanceof CreateBackgroundElement){
			type = CREATE_BACKGROUND_ELEMENT;
			return;}
		if (action instanceof LiftScrapFromBackground){
			type = LIFT_SCRAP;
			return;}
		if (action instanceof DropScrapToBackground){
			type = DROP_SCRAP;
			return;}
		if (action instanceof DuplicateScrap){
			type = DUPLICATE_SCRAP;
			return;}
		if (action instanceof EraseBGElement){
			type = ERASE_BACKGROUND_ELEMENT;
			return;}
		if (action instanceof TransformScrapToRect){
			type = TRANSFORM_SCRAP_TO_RECT;
			return;}
		if (action instanceof BGEErase){
			type = BG_ERASE;
			return;}
		if (action instanceof ChangedModel){
			type = CHANGED_MODEL;
			return;}
		if (action instanceof DropScrapToBackground){
			type = DROP_SCRAP_TO_BACKGROUND;
			return;}
		if (action instanceof ExitCanvas){
			type = EXIT_CANVAS;
			return;}
		if (action instanceof ExtendScrap){
			type = EXTEND_SCRAP;
			return;}
		if (action instanceof UndoAction){
			type = UNDO_ACTION;
			return;}
		
		type = 0;
		System.out.println("oh noes not a right snapshot type");
	}
	
	public ScrapPainter getScrap(int id) {
		PLayer layer = getLayer();
		
		for (int i = 0; i < layer.getChildrenCount(); i++) {
			if (layer.getChild(i) instanceof ScrapPainter &&
					((ScrapPainter)layer.getChild(i)).getModel().equals(id))
				return (ScrapPainter)layer.getChild(i);
		}
		return null;
	}
	
	public int getGridCell(){
		return gridCell;
	}

}		
	

/*
public boolean isNavigation(){
	if(type == VIEW_GRID || type == CHANGE_GRID_CELL){
		return true;
	}
	return false;
}*/