package ics.uci.edu.Calico.networking;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.actions.CalicoAction;
import ics.uci.edu.Calico.models.actions.CalicoActionListener;


import java.io.*;
import java.net.*;
import java.util.ArrayList;
import java.util.Hashtable;

public class ClientWorker implements Runnable {
   
	private static final long serialVersionUID = 1L;
	private CalicoCanvas canvas;
	private ObjectOutputStream out;
	private ObjectInputStream in;
	private NetworkPacket networkPacket;
	private Socket client;
	private String nickname;
	//private BuddyList buddyList;
	private SocketMultiThreadedServer server;
	private boolean firstGridUpdate = true;
	
public ClientWorker(Socket cl, BuddyList bList, SocketMultiThreadedServer server) {
	  firstGridUpdate = true;
	  client = cl;
	  out = null;
	  in = null;
	  networkPacket = null;
	  this.server = server;
  }

public ClientWorker(){
	//empty
}
  
  public void run(){
    int packetID = -1;
	
    try{
     out = new ObjectOutputStream(client.getOutputStream());
     in = new ObjectInputStream(client.getInputStream());
    } catch (IOException e) {
      System.out.println("in or out failed");
      System.exit(-1);
    }
    while(!client.isClosed()){//while client is open
      try{ 
    	  networkPacket = (NetworkPacket)in.readObject();
    	  packetID = networkPacket.getPacketID();
    	  if(packetID == networkPacket.NICKNAME){
    		  nickname = networkPacket.getNickname();
    		  server.toConsole(nickname + " connected!");
    	  }
    	  else if(packetID == networkPacket.CALICO_ACTION_ID){
    		 // System.out.println("SERVER: Calico Action w/ id received from: " + networkPacket.getNickname());
    		  server.applyAction(networkPacket.getCalicoAction(), networkPacket.getCanvasID(), networkPacket.getNickname());
    	  }
    	  else if(packetID == networkPacket.GRID_REQUEST){
    		  server.toConsole("SERVER: Grid Update Request Recieved");
    		  sendCurrentGrid();
    	  }
    	  else if(packetID == networkPacket.CANVAS_ACTIVATION_REQUEST){
    		  server.activateCanvas(networkPacket.getCanvasID(), networkPacket.getNickname());
    		  //System.out.println("SERVER: Canvas Activation Request Recieved for Canvas: " + networkPacket.getCanvasID());
    	  }
       } catch (IOException e) {
    	   	//finalize();
       } catch (ClassNotFoundException e) {
			System.out.println("Class could not be found");
			e.printStackTrace();
			}
    	} 
  }
  
  public void sendCurrentGrid(){
		  ArrayList<SerializedCell> serializedCellList = prepActiveListToSend();
		  NetworkPacket nP = new NetworkPacket(serializedCellList);
		  this.sendNetworkPacket(nP);
  }

  
  private ArrayList<SerializedCell> prepActiveListToSend(){
	 ArrayList<SerializedCell> output = new ArrayList<SerializedCell>();
	 Hashtable<Integer, ActiveCanvas> canvases = server.getActiveCanvasList();
	 ArrayList<Integer> activeIDs = server.getActiveIDs();
	 
	 
	 for(Integer aID : activeIDs){
		 ActiveCanvas aC = canvases.get(aID);
		 //if the canvases have been changed OR it is the preliminary grid request.
		 if(aC.isUpdated() || firstGridUpdate){
			 if(!aC.lastUpdatedBy().equals(nickname)){ 
				 SerializedCell sCell = new SerializedCell(aC.getPosition(),aC.getCanvas().getModel(),4);//activeIDs.size());
				 output.add(sCell);}
		 } 
	 }

	 firstGridUpdate = false;
	 server.toConsole("Sending Grid of Size: " + output.size());
	 return output;
  }
  
  
  
  public void performDrawAction(CalicoAction action){
	  
	  CalicoActionListener actionListener = null;
	  for (CalicoActionListener listener : canvas.getListeners())
		  if (listener instanceof ClientWorker)
			  actionListener = listener;
	  
	  canvas.removeCalicoActionListener(actionListener);
	  canvas.applyAction(action);
	  canvas.repaint();
	  canvas.addCalicoActionListener(actionListener); 
  }
  
  
  /**
   * 
   * @return true if successful
   */
  public boolean ping(){
	  //System.out.println("pinging: " + nickname);
	  boolean flag = true;
      try {
    	  out.writeObject(null);
	} catch (IOException e) {
		flag =  false;
		//System.out.println("nothing there");
		//isConnected = false;
	}
	return flag;
  }
  
  
  /**
   * 
   * @param nP
   * @return returns true if in error
   */
  public void sendNetworkPacket(NetworkPacket nP){
	  try {
			out.writeObject(nP);
		} catch (IOException e) {
			//finalize();
		}
  }
  
  public String getNickname(){
	  return nickname;
  }
  
  public void setNickname(String name){
	  this.nickname = name;
  }
  
  protected void finalize(){
	     try{
	         in.close();
	         out.close();
	    } catch (IOException e) {
	        System.out.println("Could not close.");
	        //System.exit(-1);
	    }
	  }
  
  
}