package ics.uci.edu.Calico.networking;

import ics.uci.edu.Calico.networking.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import ics.uci.edu.Calico.UI.CalicoOptions;

/*
PACKET STRUCTURE:

BYTE	TYPE	USAGE
----	----	------
0-3		INT		COMMAND

4-7		INT		PARAM1
8-11	INT		PARAM2
12-15	INT		PARAM3
16-19	INT		PARAM4
20-23	INT		PARAM5
24-27	INT		PARAM6
28-31	INT		PARAM7
32-35	INT		PARAM8 

36-n	BIN		BINARY DATA

*/

public class Networking2
{
	private static InetAddress serverAddress = null;
	private static int serverPort = 87871;

	private static void debug(String msg)
	{
		System.out.println("DEBUG (NET): " + msg);
	}

	public static void setServerAddress(String address) throws UnknownHostException
	{
		serverAddress = InetAddress.getByName(address);
	}
	
	
	private static byte[] intToDWord(int i)
	{
		byte[] dword = new byte[4];
		dword[0] = (byte) (i & 0x00FF);
		dword[1] = (byte) ((i >> 8) & 0x000000FF);
		dword[2] = (byte) ((i >> 16) & 0x000000FF);
		dword[3] = (byte) ((i >> 24) & 0x000000FF);
		return dword;
	}
		
	private static byte[] objectToBytes(Object obj)
	{
		try
		{
			ByteArrayOutputStream fos = new ByteArrayOutputStream();
			ObjectOutputStream out = new ObjectOutputStream(fos);
			out.writeObject(obj);
			out.close();
			return fos.toByteArray();
		}
		catch(Exception e)
		{
			return new byte[1];
		}
	}
	private static Object bytesToObject(byte[] bytes) throws Exception
	{
		ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
		ObjectInputStream in = new ObjectInputStream(bais);
		Object obj = in.readObject();
		in.close();
		return obj;
	}

	public static boolean send(CalicoCommand command, Object obj, int... params)
	{
		return sendPacket(command.ordinal(), obj, params);
	}
	public static boolean send(int command, Object obj, int... params)
	{
		return sendPacket(command, obj, params);
	}

	private static boolean sendPacket(int command, Object obj, int[] params)
	{
		try
		{
			int packetSize = 36;
			byte[] objBytes = objectToBytes(obj);

			packetSize = packetSize + objBytes.length;
			
			byte[] data = new byte[packetSize];
			int packetIndex = 0;
			byte[] datac = intToDWord(command);
			for(int c=0;c < datac.length;c++)
			{
				data[packetIndex] = datac[c];
				packetIndex++;
			}


			for(int i=0;i<params.length;i++)
			{
				byte[] datap = intToDWord(params[i]);
				for(int p=0;p < datap.length;p++)
				{
					data[packetIndex] = datap[p];
					packetIndex++;
				}
			}

			for(int o=0;o<objBytes.length;o++)
			{
				data[packetIndex] = objBytes[o];
				packetIndex++;
			}


			DatagramSocket socket = new DatagramSocket();
			DatagramPacket packet = new DatagramPacket(data, 0, data.length, serverAddress, serverPort);
			socket.send(packet);
			socket.close();
			return true;
		}
		catch(Exception e)
		{
			debug("Apparently, the server has exploded. We are deeply sorry.");
			return false;
		}
	}

	// need a receive function
	
}
