package ics.uci.edu.Calico.networking;


import ics.uci.edu.Calico.UI.components.CalicoGrid;
import ics.uci.edu.Calico.models.ObjectHandlerModel;
import ics.uci.edu.Calico.models.actions.CalicoActionEvent;
import ics.uci.edu.Calico.UI.CalicoOptions;

import javax.swing.*;

import java.io.*;
import java.net.*;

public class SocketClient{

	private static final long serialVersionUID = 1L;
	private Socket socket;
	private ObjectOutputStream out;
	private ObjectInputStream in;
	private boolean connected = false;
	private boolean inError = false;
	private NetworkPacket networkPacket;
	private Networking networking;
	private int currentCanvasID;
	private String nickname;
	
  SocketClient(Networking networking){ 
       socket = null;
       out = null;
       in = null;
       networkPacket = null;
       this.networking = networking;
       connected = false;
       }
  
  public void setCanvas(int ID){
	  currentCanvasID = ID;
  }
      
   public boolean isConnected(){
	   return connected;
   }

   public void sendCanvas(ObjectHandlerModel oHM){
	   try {
			NetworkPacket nP = new NetworkPacket(oHM);
			out.writeObject(nP);
	 	} catch (IOException e) {
			connected = false;
		}
   }
   
   //when action occurs, write action to output stream
   public void fireAction(CalicoActionEvent action, String nickname){
	   if(isConnected()){
		   networkPacket = new NetworkPacket(action.getAction(),currentCanvasID, nickname);
		          try {
		        	  out.writeObject(networkPacket);
				} catch (IOException e) {
					message("Warning: Server not active");
					connected = false;
					finalize();
					networking.drawConnectionFrame();
				}
	   	}
   }
   
   public void sendNetworkPacket(NetworkPacket nP){
	   try {
		   int packetID = nP.getPacketID();
		   if(connected){
			   if(packetID == nP.CANVAS_ACTIVATION_REQUEST)
				   System.out.println("Client: Sending Activation Request");
			
			   out.writeObject(nP);
			}
	 	} catch (IOException e) {
	 		message("Warning: Server not active");
			connected = false;
			finalize();
			networking.drawConnectionFrame();
		}
   }
  
  
  public void connect(String serverAddress, String nickname){
	  try {
				socket = new Socket(serverAddress, CalicoOptions.network_port);
				connected = true;
				message("Connected!");
				networking.removeConnectFrame();
				inError = false;
			} catch (UnknownHostException e) {
				message("Could not find server");
				connected = false;
				inError = true;
			} catch (IOException e) {
				message("Could not find server");
				connected = false;
				inError = true;
			}
			
	//if not in error	
			if(!inError){
    	 	try {
    	 		this.nickname = nickname;
				out = new ObjectOutputStream(socket.getOutputStream());
				in = new ObjectInputStream(socket.getInputStream());
				sendNetworkPacket(new NetworkPacket(nickname));
				//request for current CalicoGrid!
				char temp = 'a';
				sendNetworkPacket(new NetworkPacket(temp));
				
				ClientInputListener c = new ClientInputListener(in,this);
	    		Thread t = new Thread(c);
	    		t.start();
				
    	 	} catch (IOException e) {
				connected = false;
				e.printStackTrace();
			}
    	 
	}
			
 }
  
 
 
  

  
  /**
   * depreciated
   */
  /*public void setTimer(){
	  System.out.println("TIMER STARTED");
	   int delay = 5000; //milliseconds(1000ms = 1s)
	   ActionListener taskPerformer = new ActionListener() {
	       public void actionPerformed(ActionEvent evt){ 
	    	   sendCurrentCanvas();
	       }
	   };
	   new Timer(delay, taskPerformer).start();
  }
  */
 
  
  public void save(){
	  networking.getMenu().saveCanvas();
  }
  
  public CalicoGrid getCalicoGrid(){
	  return networking.getCalicoGrid();
  }
  
  public String getNickname(){
	  return nickname;
  }
 
  
  
  protected void finalize(){
	 
	    connected = false; 
	  	try{
	         out.close();
	         in.close();
	         message("Disconnected!");
	    } catch (IOException e) {
	        System.out.println("Could not close.");
	    }
	  }
  
 
   private void message(String message){
	   JFrame errorFrame = new JFrame();
	   JOptionPane.showMessageDialog(errorFrame,
			    message,
			    "Client Message",
			    JOptionPane.INFORMATION_MESSAGE);
	   
   }
   
}
