package ics.uci.edu.Calico.networking;

import ics.uci.edu.Calico.UI.CalicoCanvas;
import ics.uci.edu.Calico.models.actions.CalicoAction;
import ics.uci.edu.Calico.UI.CalicoOptions;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.net.*;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.Timer;

public class SocketMultiThreadedServer extends JFrame implements Runnable{
	
	private static final long serialVersionUID = 1L;
	private ServerSocket server;
	private Vector<ClientWorker> clientWorkerList;
	private BuddyList buddyList;
	private Hashtable<Integer, ActiveCanvas> activeCanvases;
	private ArrayList<Integer> activeCanvasIDs;
	private StandAloneServer UI;
	
   public SocketMultiThreadedServer(StandAloneServer UI){
	   this.UI = UI;
	   activeCanvasIDs = new ArrayList<Integer>();
	   activeCanvases = new Hashtable<Integer, ActiveCanvas>();
	   server = null;
	   clientWorkerList = new Vector<ClientWorker>();
	   //buddyList = new BuddyList();
   } 
   
   public void listenSocket(){
	   int counter = 0;
	   System.out.println("Starting Server");
	    try{
	    	server = new ServerSocket(CalicoOptions.network_port);//server is a socket made through port 6112
	    } catch (IOException e) {//if port cannot be found/port-failed exit
	      message("Server Cannot Connect\nport "+CalicoOptions.network_port+" is in use");
	      System.exit(-1);
	    }
	    	//TODO
	    	//message("Server Started");
		    //READ IN DATA
		    while(true){
		    	ClientWorker w;
		      try {
			    	w = new ClientWorker(server.accept(), buddyList, this);
			    	clientWorkerList.add(w);
			    	Thread t = new Thread(w);
			    	t.start();
			    	counter++;
			    	if(counter==1)
			    		setTimer();
				} catch (IOException e) {
				System.out.println("IO connection could not be made");
				e.printStackTrace();
		      	}
		    }
	  }
   
   public void activateCanvas(Integer ID, String nickname){
	  CalicoCanvas canvas = new CalicoCanvas();
	
	  if(!activeCanvases.containsKey(ID)){
		  ActiveCanvas aCanvas = new ActiveCanvas(ID, canvas);
		  aCanvas.addUser(nickname);
		  aCanvas.updateCanvas(nickname);
		  
		  activeCanvases.put(ID, aCanvas);
		  activeCanvasIDs.add(ID);
		  UI.toConsole("Canvas " + ID + " Activated!");
		  UI.addToFrame(ID);
		  //UI.addUser(ID, user);
	  }
	  else{
		  activeCanvases.get(ID).addUser(nickname);
		  //UI.addUser(ID, user);
	  }
   }

   public void applyAction(CalicoAction action, Integer id, String nickname){
	   
	   ActiveCanvas aC = activeCanvases.get(id);
	   aC.getCanvas().applyAction(action);
	   aC.updateCanvas(nickname);
	   UI.markAsUpdated(id);
	   sendToUsers(nickname, action, id);
	   UI.toConsole("ACTION APPLIED TO CANVAS " + id + " and marked as updated");
   }
   

   
   public void sendToUsers(String nickname, CalicoAction action, Integer id){
	   
	   for(ClientWorker cW :  clientWorkerList){
		   cW.sendNetworkPacket(new NetworkPacket(action, id, nickname)); 
		  	}
   }
   
   
   public Hashtable<Integer,ActiveCanvas> getActiveCanvasList(){
	   return activeCanvases;
   }
   
   public ArrayList<Integer> getActiveIDs(){
	   return activeCanvasIDs;
   }
   
   
   public void pingAll(){
	   ClientWorker toBeDeleted = new ClientWorker();
	   toBeDeleted.setNickname("@!#@$!#@!");
	   
	   for(ClientWorker cW : clientWorkerList){
			//if not successful, delete!  
		   if(!cW.ping()){
			   //System.out.println(cW.getNickname() + " to be deleted");
			   toBeDeleted = cW;}
		   }
	   
	   if(!toBeDeleted.getNickname().equals("@!#@$!#@!")){
		   //System.out.println(toBeDeleted.getNickname() + " has been deleted");
		   UI.removeUser(toBeDeleted.getNickname());
		   clientWorkerList.remove(toBeDeleted);
		   toBeDeleted.finalize();
		   UI.toConsole(toBeDeleted.getNickname() + " Disconnected!");
	   }
	   
	   if(clientWorkerList.size()==0){
		   
	   }
	   
   }
   
   
   private void updateAll(){  
	   for(ClientWorker cW : clientWorkerList){
		  cW.sendCurrentGrid();
	   }
	   
	   //TODO perhaps mark all ActiveClients as not updated. Good idea.. thanks me
	   for(Integer aID : activeCanvasIDs){
			ActiveCanvas aC = activeCanvases.get(aID);
			UI.markAsNotUpdated(aID);
			aC.noLongerUpdated(); 
		 }
   }

   
   private void setTimer(){
	   UI.toConsole("Set Timer");
	   int delay = 5000; //milliseconds(1000ms = 1s)
	   ActionListener taskPerformer = new ActionListener() {
	       public void actionPerformed(ActionEvent evt) {
	    	   pingAll();	 
	    	   updateAll();
	       }
	   };
	   new Timer(delay, taskPerformer).start();
   }

  protected void finalize(){
//	Clean up 
	     try{
	         server.close();
	    } catch (IOException e) {
	        System.out.println("Could not close.");
	        System.exit(-1);
	    }
	  }
  
   private void message(String message){
	   JFrame errorFrame = new JFrame();
	   JOptionPane.showMessageDialog(errorFrame,
			    message,
			    "Server Message",
			    JOptionPane.INFORMATION_MESSAGE);
   }

   public void toConsole(String text){
	   UI.toConsole(text);
   }

	public void run() {
		listenSocket();
	}
}





